

import copy, time, StringIO
import unittest
from datetime import datetime
from datetime import date

from nive.utils.dataPool2.structure import *

from nive.tests import __local

from nive.utils.dataPool2.tests import test_Base


ftypes = {}
ftypes[u"data2"] = {u"fstr":"string",
                    u"ftext":"text", 
                    u"ftime":"timestamp", 
                    u"fmselection":"mselection",
                    u"fmcheckboxes":"mcheckboxes", 
                    u"furllist":"urllist", 
                    u"funitlist":"unitlist",
                    u"fbool":"bool",
                    u"fjson":"json"}

ftypes[u"pool_meta"] = {}
for i in test_Base.SystemFlds:
    ftypes[u"pool_meta"][i["id"]] = i

class StructureTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_set1(self):
        structure = PoolStructure(structure=test_Base.struct, 
                                  fieldtypes=ftypes, 
                                  stdMeta=[u"id",u"pool_type"])
        self.assert_(structure.get(u"pool_meta"))
        self.assert_(len(structure.get(u"pool_meta")) == len(test_Base.struct[u"pool_meta"]))
        self.assert_(len(structure.get(u"data1"))==len(test_Base.struct[u"data1"]))
        self.assert_(len(structure.get(u"data2"))==len(test_Base.struct[u"data2"]))
        self.assert_(len(structure.stdMeta)==2)
        self.assert_(structure.fieldtypes[u"data2"][u"fstr"]=="string")
        self.assert_(structure.codepage==u"utf-8")

    def test_set2(self):
        structure = PoolStructure()
        structure.Init(structure=test_Base.struct, 
                       fieldtypes=ftypes, 
                       codepage="latin-1")
        self.assert_(structure.get(u"pool_meta"))
        self.assert_(len(structure.get(u"pool_meta")) == len(test_Base.struct[u"pool_meta"]))
        self.assert_(len(structure.get(u"data1"))==len(test_Base.struct[u"data1"]))
        self.assert_(len(structure.get(u"data2"))==len(test_Base.struct[u"data2"]))
        self.assert_(len(structure.stdMeta)==0)
        self.assert_(structure.fieldtypes[u"data2"][u"fstr"]=="string")
        self.assert_(structure.codepage==u"latin-1")
        
    def test_set3(self):
        structure = PoolStructure()
        structure.Init(structure={u"pool_meta": [], u"data1": [], u"data2": []}, 
                       fieldtypes=ftypes, 
                       codepage="latin-1")
        self.assert_(structure.get(u"pool_meta"))
        self.assert_(len(structure.get(u"pool_meta"))==2)
        self.assert_(len(structure.get(u"data1"))==0)
        self.assert_(len(structure.get(u"data2"))==0)
        


    def test_empty(self):
        structure = PoolStructure()
        self.assert_(structure.IsEmpty())


    def test_func(self):
        structure = PoolStructure(structure=test_Base.struct, 
                                  fieldtypes=ftypes, 
                                  stdMeta=[u"id",u"pool_type"])
        self.assertFalse(structure.IsEmpty())
        
        self.assert_(structure.get("pool_meta"))
        self.assert_(structure.get("none","aaa")=="aaa")
        self.assert_(structure["pool_meta"])
        self.assert_(structure["data1"])
        self.assert_(structure["data2"])
        self.assert_(structure.has_key("data2"))
        self.assert_(len(structure.keys())==3)
        
        

        
class ConversionTest(unittest.TestCase):

    def setUp(self):
        self.structure = PoolStructure(structure=test_Base.struct, 
                                       fieldtypes=ftypes, 
                                       stdMeta=[u"id",u"pool_type"])

    def tearDown(self):
        pass


    def test_serialize_notype(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"somevalue", 123)==123)
        self.assert_(isinstance(self.structure.serialize(u"pool_meta", u"somevalue", "123"), unicode))
        value = datetime.now()
        self.assert_(self.structure.serialize(u"pool_meta", u"somevalue", value)==value.strftime("%Y-%m-%d %H:%M:%S"))
        value = ("aaa","bbb")
        self.assert_(self.structure.serialize(u"pool_meta", u"somevalue", value).startswith(u"_json_"))
        value = (u"aaa",u"bbb")
        self.assert_(self.structure.serialize(u"pool_meta", u"somevalue", value).startswith(u"_json_"))
        value = [1,2,3]
        self.assert_(self.structure.serialize(u"pool_meta", u"somevalue", value).startswith(u"_json_"))
        
    def test_se_mselection(self):
        v = {u"id":u"123", u"pool_sort":u"123.12", u"pool_wfa":["value"], u"somevalue": "test"}
        values = self.structure.serialize(u"pool_meta", None, v)
        self.assert_(values[u"id"]==123)
        self.assert_(values[u"pool_sort"]==123.12)
        self.assert_(values[u"pool_wfa"]==u"value")        

    def test_se_number(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"id", 123)==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", u"123")==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", "123")==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", 123.12)==123)

    def test_se_float(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", 123)==123.0)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", u"123.12")==123.12)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", "123.0")==123.0)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", 123.12)==123.12)

    def test_se_date(self):
        value = datetime.now()
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_change", value)==unicode(value))
        value = date.today()
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_create", value)==unicode(value))
        value = time.time()
        self.assert_(self.structure.serialize(u"data2", u"ftime", value)==unicode(value))

    def test_se_list(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_wfa", u"value")==u"value")
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_wfa", ["value"])=="value")
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_wfa", ())=="")

    def test_se_mlist(self):
        self.assert_(self.structure.serialize(u"data2", u"fmselection", u"value"))
        self.assert_(self.structure.serialize(u"data2", u"fmselection", [u"value"]))
        self.assert_(self.structure.serialize(u"data2", u"fmselection", ("value",)))
        self.assertFalse(self.structure.serialize(u"data2", u"fmselection", u""))
        
        self.assert_(self.structure.serialize(u"data2", u"mcheckboxes", u"value"))
        self.assert_(self.structure.serialize(u"data2", u"furllist", u"value"))
        self.assert_(self.structure.serialize(u"data2", u"funitlist", u"value"))

    def test_se_bool(self):
        self.assert_(self.structure.serialize(u"data2", u"fbool", u"true")==1)
        self.assert_(self.structure.serialize(u"data2", u"fbool", u"false")==0)
        self.assert_(self.structure.serialize(u"data2", u"fbool", True)==1)
        self.assert_(self.structure.serialize(u"data2", u"fbool", False)==0)
        self.assert_(self.structure.serialize(u"data2", u"fbool", u"True")==1)
        self.assert_(self.structure.serialize(u"data2", u"fbool", u"False")==0)
        self.assert_(self.structure.serialize(u"data2", u"fbool", ("???",))==0)

    def test_se_json(self):
        self.assert_(self.structure.serialize(u"data2", u"fjson", {"a":123,"b":"aaa"}))
        self.assert_(json.loads(self.structure.serialize(u"data2", u"fjson", {"a":123,"b":"aaa"}))["a"]==123)
        self.assert_(json.loads(self.structure.serialize(u"data2", u"fjson", {"a":123,"b":"aaa"}))["b"]==u"aaa")
    
    
    def test_deserialize_notype(self):
        value = u"_json_"+json.dumps(("aaa","bbb"))
        self.assert_(self.structure.deserialize(u"pool_meta", u"somevalue", value)[0]==u"aaa")
        self.assert_(self.structure.deserialize(u"pool_meta", u"somevalue", "somevalue")==u"somevalue")
        
    def test_ds_mselection(self):
        v = {u"fmselection": json.dumps(["aaa","bbb"]),u"furllist":json.dumps(["aaa","bbb"]), u"somevalue": "test"}
        values = self.structure.deserialize(u"data2", None, v)
        self.assert_(values[u"fmselection"][0]=="aaa")
        self.assert_(values[u"furllist"][0]=="aaa")

    def test_ds_date(self):
        value = datetime.now()
        x=self.structure.deserialize(u"pool_meta", u"pool_change", unicode(value))
        self.assert_(x.strftime("%Y-%m-%d %H:%M:%S")==value.strftime("%Y-%m-%d %H:%M:%S"))
        value = date.today()
        x=self.structure.deserialize(u"pool_meta", u"pool_create", unicode(value))
        self.assert_(x.strftime("%Y-%m-%d")==value.strftime("%Y-%m-%d"))
        value = time.time()
        self.assert_(self.structure.deserialize(u"data2", u"ftime", value))

    def test_ds_mselection(self):
        self.assert_(self.structure.deserialize(u"data2", u"fmselection", json.dumps(["aaa","bbb"]))[0]=="aaa")
        self.assert_(self.structure.deserialize(u"data2", u"fmcheckboxes", json.dumps(["aaa","bbb"]))[0]=="aaa")
        self.assert_(self.structure.deserialize(u"data2", u"furllist", json.dumps(["aaa","bbb"]))[0]=="aaa")
        self.assert_(self.structure.deserialize(u"data2", u"funitlist", json.dumps(["aaa","bbb"]))[0]=="aaa")
        
    def test_ds_json(self):
        self.assert_(self.structure.deserialize(u"data2", u"fjson", json.dumps(["aaa","bbb"]))[0]=="aaa")



def seCallback(value, field):
    return value.swapcase()

def deCallback(value, field):
    return value.capitalize()



class CallbackTest(unittest.TestCase):

    def setUp(self):
        self.structure = PoolStructure(structure=test_Base.struct, 
                                       fieldtypes=ftypes, 
                                       stdMeta=[u"id",u"pool_type"])
        self.structure.serializeCallbacks = {"string": seCallback}
        self.structure.deserializeCallbacks = {"string": deCallback}


    def tearDown(self):
        pass


    def test_serialize_callback(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"title", u"somevalue")==u"SOMEVALUE")
        self.assert_(self.structure.deserialize(u"pool_meta", u"title", u"somevalue")==u"Somevalue")

        
    def test_se_mselection(self):
        v = {u"id":u"123", u"pool_sort":u"123.12", u"pool_wfa":["value"], u"somevalue": "test"}
        values = self.structure.serialize(u"pool_meta", None, v)
        self.assert_(values[u"id"]==123)
        self.assert_(values[u"pool_sort"]==123.12)
        self.assert_(values[u"pool_wfa"]==u"value")        

    def test_se_number(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"id", 123)==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", u"123")==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", "123")==123)
        self.assert_(self.structure.serialize(u"pool_meta", u"id", 123.12)==123)

    def test_se_float(self):
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", 123)==123.0)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", u"123.12")==123.12)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", "123.0")==123.0)
        self.assert_(self.structure.serialize(u"pool_meta", u"pool_sort", 123.12)==123.12)

        


#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
nive user database 
--------------------

You can specify a admin user on configuration level as `admin`. The admin user works without 
database connection.

The system admin for notification mails can be specified as `systemAdmin`.
::

    configuration.admin = {"name": "admin", "password": "adminpass", "email": "admin@domain.com"}
    configuration.systemAdmin = (u"email", u"display name")

"""

from nive.definitions import implements, AppConf, FieldConf, GroupConf, IUserDatabase, ILocalGroups
from nive.security import Allow, Deny, Everyone, ALL_PERMISSIONS, remember, forget
from nive.components.objects.base import ApplicationBase
from nive.i18n import _

#@nive_module
configuration = AppConf()
configuration.id = "userdb"
configuration.title = _(u"Users")
configuration.context = "nive.userdb.app.UserDB"
configuration.loginByEmail = False

# configuration.systemAdmin = (u"email", u"display name")
# configuration.admin = {"name": "admin", "password": "adminpass", "email": "admin@domain.com"}

configuration.modules = [
    "nive.userdb.root", 
    "nive.userdb.user", 
    # session user cache
    "nive.components.extensions.sessionuser",
    # user administration
    "nive.userdb.useradmin", 
    # tools
    "nive.components.tools.dbStructureUpdater", 
    # administration and persistence
    "nive.adminview",
    #"nive.components.extensions.persistence.dbPersistenceConfiguration"
]

configuration.acl= [
    (Allow, Everyone, 'view'),
    (Allow, Everyone, 'updateuser'),
    (Allow, "group:useradmin", 'signup'), 
    (Allow, "group:useradmin", 'manage users'),
    (Allow, "group:admin", ALL_PERMISSIONS),
]

configuration.groups = [ 
    GroupConf(id="group:useradmin", name="group:useradmin"),
]


class UserDB(ApplicationBase):
    """
    """
    implements(IUserDatabase)


        
    def Groupfinder(self, userid, request=None, context=None):
        """
        returns the list of groups assigned to the user 
        """
        if request:
            try:
                user = request.environ["authenticated_user"]
            except:
                user = self.root().GetUser(userid)
                request.environ["authenticated_user"] = user
                def remove_user(request):
                    if "authenticated_user" in request.environ:
                        del request.environ["authenticated_user"]
                request.add_finished_callback(remove_user)
        else:
                user = self.root().GetUser(userid)
        if not user:
            return None

        # users groups or empty list
        groups = user.groups or ()

        # lookup context for local roles
        if not context and hasattr(request, "context"):
            context = request.context
        if context and ILocalGroups.providedBy(context):
            local = context.GetLocalGroups(userid, user=user)
            if not groups:
                return local
            return tuple(list(groups)+list(local))
        return groups


    def RememberLogin(self, request, user):
        """
        add login info to cookies or session. 
        """
        if not hasattr(request.response, "headerlist"):
            request.response.headerlist = []
        headers = remember(request, user)
        request.response.headerlist += list(headers)


    def ForgetLogin(self, request, url=None):
        """
        removes login info from cookies and session
        """
        if not hasattr(request.response, "headerlist"):
            setattr(request.response, "headerlist", [])
        headers = forget(request)
        request.response.headerlist += list(headers)
        #request.authenticate




    def AuthenticatedUser(self, request):
        # bw 0.9.6. removed in next version.
        name = self.UserName(request)
        return self.GetRoot().GetUserByName(name)

    def AuthenticatedUserName(self, request):
        # bw 0.9.6. removed in next version.
        return authenticated_userid(request)    





from nive.definitions import ModuleConf

configuration = ModuleConf(
    id="cms-view-module",
    name="Meta package to load cms editor components",
    modules=(
        "nive.cms.cmsview.cmsroot", 
        "nive.cms.cmsview.view", 
        "nive.cms.cmsview.admin", 
        "nive.components.reform.reformed"
    ),
)



import time
import unittest

from nive.security import User
from nive.definitions import *

import db_app

# -----------------------------------------------------------------

class SearchTest_db(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_db()
        r=self.app.root()
    
        # create three levels of entries
        level1 = 5
        level2 = 5
        level3_1 = 5
        level3_2 = 5
        c=r
        ids=[]
        n=0
        user = User(u"test")
        for i in range(0,level1):
            o=db_app.createObj1(c)
            n+=1
            ids.append(o.id)
            for i2 in range(0,level2):
                o2=db_app.createObj1(o)
                n+=1
                for i3 in range(0,level3_1):
                    db_app.createObj1(o2)
                    n+=1
                for i3 in range(0,level3_2):
                    o4 = db_app.createObj2(o2)
                    id = o4.id
                    n+=1
        self.ids = ids
        self.lastid = id

    def tearDown(self):
        user = User(u"test")
        r=self.app.root()
        for id in self.ids:
            r.Delete(id, user=user)
        self.app.Close()

    
    def test_search(self):
        r = self.app.root()
        #test_tree
        self.assert_(len(r.TreeParentIDs(self.lastid))==2)
        self.assert_(len(r.TreeParentTitles(self.lastid))==2)
        self.assert_(len(r.TreeParentIDs(self.ids[3]))==0)
        self.assert_(len(r.TreeParentTitles(self.ids[3]))==0)

        #test_select
        self.assert_(r.Select())
        self.assert_(r.SelectDict())
        self.assert_(r.Select(pool_type="type1"))
        self.assert_(r.SelectDict(pool_type="type1"))
        self.assertFalse(r.Select(parameter={"pool_type": "type"}, operators={"pool_type": "="}, sort="id", ascending = 0))
        self.assertFalse(r.SelectDict(parameter={"pool_type": "type"}, operators={"pool_type": "="}, sort="id", ascending = 0))
        self.assert_(r.Select(parameter={"pool_type": "type"}, operators={"pool_type": "LIKE"}, sort="id", ascending = 0))
        self.assert_(r.SelectDict(parameter={"pool_type": "type"}, operators={"pool_type": "LIKE"}, sort="id", ascending = 0))
        self.assert_(r.Select(pool_type="type1", parameter={}, fields=["id","title","ftext","fnumber","fdate"], start=10, max=1))
        self.assert_(r.SelectDict(pool_type="type1", parameter={}, fields=["id","title","ftext","fnumber","fdate"], start=10, max=1))
        self.assert_(r.Select(parameter={"pool_type": "notype", "title": "others"}, logicalOperator="or", operators={"pool_type":"LIKE", "title":"LIKE"}))
        self.assert_(r.SelectDict(parameter={"pool_type": "notype", "title": "others"}, logicalOperator="or", operators={"pool_type":"LIKE", "title":"LIKE"}))
        self.assertFalse(r.Select(parameter={"pool_type": "notype", "title": "notitle"}, logicalOperator="or", operators={"pool_type":"LIKE", "title":"LIKE"}))
        self.assertFalse(r.SelectDict(parameter={"pool_type": "notype", "title": "notitle"}, logicalOperator="or", operators={"pool_type":"LIKE", "title":"LIKE"}))
        self.assert_(r.Select(groupby="title"))
        self.assert_(r.SelectDict(groupby="title"))
        self.assert_(r.Select(condition="id > 23"))
        self.assert_(r.SelectDict(condition="id > 23"))

        #test_codelists
        pool_type="type1"
        name_field="title"
        self.assert_(r.GetEntriesAsCodeList(pool_type, name_field))
        self.assert_(r.GetEntriesAsCodeList2(name_field))
        self.assert_(r.GetGroupAsCodeList(pool_type, name_field))
        self.assert_(r.GetGroupAsCodeList2(name_field))
        parameter = {"pool_state":1}
        operators = {"pool_state":"<="}
        self.assert_(r.GetEntriesAsCodeList(pool_type, name_field, parameter= parameter, operators = operators))
        self.assert_(r.GetEntriesAsCodeList2(name_field, parameter= parameter, operators = operators))
        self.assert_(r.GetGroupAsCodeList(pool_type, name_field, parameter= parameter, operators = operators))
        self.assert_(r.GetGroupAsCodeList2(name_field, parameter= parameter, operators = operators))

        #test_conversion
        pool_type="type1"
        r.FilenameToID("number1")
        r.IDToFilename(self.lastid)
        dataref = r.Select(parameter={"pool_type": pool_type}, fields=["id","pool_dataref"], max=1)[0]
        self.assert_(r.ConvertDatarefToID(pool_type, dataref[1])==dataref[0])
        self.assert_(r.GetMaxID())

        #test_refs
        self.assert_(r.GetReferences(35))
        self.assert_(r.GetReferences(35, types=["type1"]))

        #test_search
        r = self.app.root()
        parameter = {"pool_state":1}
        operators = {"pool_state":"<="}
        pool_type = "type1"
        fields1 = ["id","title","pool_state","pool_unitref","pool_wfa"]
        fields2 = ["id","title","pool_state","pool_unitref","pool_wfa","ftext"]
        fields3 = ["id","ftext"]

        d=r.Select(fields=["-count(*)"])
        self.assert_(r.Select(parameter=parameter, fields=fields1, start=0, max=100))
        self.assert_(r.Select(pool_type=pool_type, parameter=parameter, fields=fields2, start = 0, max=100, operators=operators))
        self.assert_(r.SelectDict(parameter=parameter, fields=fields1, start=0, max=100))
        self.assert_(r.SelectDict(pool_type=pool_type, parameter=parameter, fields=fields2, start = 0, max=100, operators=operators))

        self.assert_(r.Search(parameter, fields = fields1, sort = "title", ascending = 1, start = 0, max = 100)["count"])
        self.assert_(r.SearchType(pool_type, parameter, fields = fields2, sort = "title", ascending = 1, start = 0, max = 100, operators=operators)["count"])
        self.assert_(r.SearchData(pool_type, {}, fields = fields3, sort = "id", ascending = 1, start = 0, max = 100, operators={})["count"])
        self.assert_(r.SearchFulltext("text", fields = fields1, sort = "title", ascending = 1, start = 0, max = 300)) #["count"]
        self.assert_(r.SearchFulltextType(pool_type, "text", fields = fields2, sort = "title", ascending = 1, start = 0, max = 300))   #["count"]
        r.SearchFilename("file1.txt", parameter, fields = [], sort = "title", ascending = 1, start = 0, max = 100, operators=operators)
        
        #test_listitems
        self.assert_(len(r.LoadListItems(self.app.GetFld("pool_category"), obj=None, pool_type=None, force=False))==2)
        self.assert_(len(r.LoadListItems(self.app.GetFld("pool_type"), obj=None, pool_type=None, force=True))==3)
        #self.assert_(r.LoadListItems(self.app.GetFld("pool_groups"), obj=None, pool_type=None, force=True))
        r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"users"}))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"groups"})))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"languages"})))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"countries"})))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"types"})))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"type:type1"})))
        self.assert_(r.LoadListItems(FieldConf(id="test",datatype="list",settings={"codelist":"meta"})))



if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import json
import os
from datetime import datetime

from pyramid.path import DottedNameResolver
from pyramid.path import AssetResolver
from pyramid.path import caller_package


from nive.definitions import IAppConf, IDatabaseConf, IFieldConf, IRootConf, IObjectConf, IViewModuleConf
from nive.definitions import IViewConf, IToolConf, IPortalConf, IGroupConf, ICategoryConf, IModuleConf
from nive.definitions import IWidgetConf, IWfProcessConf, IWfStateConf, IWfTransitionConf, IConf
from nive.definitions import baseConf
from nive.definitions import implements, ConfigurationError
from nive.utils.dataPool2.files import File
from nive.utils.utils import LoadFromFile


def ResolveName(name, base=None, raiseExcp=True):
    """
    Lookup python object by dotted python name.
    Wraps pyramid.DottedNameResolver.
    
    returns object or None
    """
    if not name:
        return None
    if not isinstance(name, basestring):
        return name
    if not base:
        base = caller_package()
    if not raiseExcp:
        d = DottedNameResolver(base)
        return d.maybe_resolve(name)
    d = DottedNameResolver(base)
    return d.resolve(name)


def ResolveAsset(name, base=None, raiseExcp=True):
    """
    Lookup asset path (template, json or any other file) and returns asset 
    descriptor object or None.
    """
    if not name:
        return None
    if not isinstance(name, basestring):
        return name
    if not base:
        base = caller_package()
    if name.startswith("./"):
        # use relative file system path
        name = os.getcwd()+name[1:]
    if not raiseExcp:
        try:
            d = AssetResolver(base)
            return d.resolve(name)
        except:
            return None
    d = AssetResolver(base)
    return d.resolve(name)

    
def ResolveConfiguration(conf, base=None):
    """
    Lookup configuration object by dotted python name. Returns interface and configuration object.
    Extends pyramid.DottedNameResolver with .json file support for configuration objects.
    
    Supports the following cases:
    
    - Path and file name to .json file. requires `type` set to one of the 
      configuration types: 
      *AppConf, FieldConf, DatabaseConf, RootConf, ObjectConf, ViewModuleConf, ViewConf, ToolConf, 
      GroupConf, CategoryConf*
    - Dotted python name for configuration object including attribute name of configuration instance.
    - Dotted python name for object. Uses the convention to load the configuration from the 
      'configuration' attribute of the referenced object.
    - Configuration instance. Will just return it.
    
    returns Interface, configuration
    """
    # string instance
    if isinstance(conf, basestring):
        if not base:
            base = caller_package()
        # json file
        if conf.find(".json")!= -1:
            path = ResolveAsset(conf, base)
            s = LoadFromFile(path.abspath())
            conf = json.loads(s)
        # resolve attribute name
        elif conf:
            c = ResolveName(conf, base=base)
            if hasattr(c, "configuration"):
                conf = c.configuration
            else:
                conf = c

    # dict instance
    if isinstance(conf, dict):
        # load by interface
        if not "type" in conf:
            raise TypeError, "Configuration type not defined"
        c = ResolveName(conf["type"], base="nive")
        del conf["type"]
        conf = c(**conf)

    # module and not configuration
    if not IConf.providedBy(conf):
        if hasattr(conf, "configuration"):
            conf = conf.configuration
        
    # object instance
    if IAppConf.providedBy(conf): return IAppConf, conf
    if IDatabaseConf.providedBy(conf): return IDatabaseConf, conf
    if IFieldConf.providedBy(conf): return IFieldConf, conf
    if IRootConf.providedBy(conf): return IRootConf, conf
    if IObjectConf.providedBy(conf): return IObjectConf, conf
    if IViewModuleConf.providedBy(conf): return IViewModuleConf, conf
    if IViewConf.providedBy(conf): return IViewConf, conf
    if IToolConf.providedBy(conf): return IToolConf, conf
    if IPortalConf.providedBy(conf): return IPortalConf, conf
    if IGroupConf.providedBy(conf): return IGroupConf, conf
    if ICategoryConf.providedBy(conf): return ICategoryConf, conf
    if IModuleConf.providedBy(conf): return IModuleConf, conf
    if IWidgetConf.providedBy(conf): return IWidgetConf, conf
    if IWfProcessConf.providedBy(conf): return IWfProcessConf, conf
    if IWfStateConf.providedBy(conf): return IWfStateConf, conf
    if IWfTransitionConf.providedBy(conf): return IWfTransitionConf, conf
    if IConf.providedBy(conf): return IConf, conf
    return None, conf
    #raise TypeError, "Unknown configuration object: %s" % (str(conf))
    
    
def LoadConfiguration(conf, base=None):
    """
    same as ResolveConfiguration except only the configuration object is
    returned
    """
    if not base:
        base = caller_package()
    i,c = ResolveConfiguration(conf, base)
    return c


def FormatConfTestFailure(report, fmt="text"):
    """
    Format configuration test() failure
    
    returns string
    """
    v=[]
    for r in report:
        v+= u"-----------------------------------------------------------------------------------\r\n"
        v+= str(r[0]) + " " + r[1] + "\r\n"
        v+= u"-----------------------------------------------------------------------------------\r\n"
        for d in r[2].__dict__.items():
            a = d[1]
            if a==None:
                try:
                    a = r[2].parent.get(d[0])
                except:
                    pass
            v+= str(d[0])+u":  "+str(a)+u"\r\n"
        v+= u"\r\n"
    return "".join(v)


class ConfEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, baseConf):
            values = {}
            for k in obj:
                values[k] = obj[k]
            return values
        return json.JSONEncoder.default(self, obj)

class ConfDecoder(object):
    def decode(self, jsonstring):
        def object_hook(obj):
            if isinstance(obj, dict):
                try:
                    confclass = obj["ccc"]
                except:
                    return obj
                   
                if not confclass:
                    raise ConfigurationError, "Configuration class not found (ccc)"
                conf = ResolveName(confclass, base="nive")(**obj)
                return conf
            return obj
        return json.JSONDecoder(object_hook=object_hook).decode(jsonstring) 

        
def DumpJSONConf(conf):
    # dump configuration to json
    values = {}
    for k in conf:
        v = conf[k]
        if isinstance(v, baseConf):
            values[k] = DumpJSONConf(v)
        elif isinstance(v, datetime):
            values[k] = str(v)
        else:
            values[k] = v
    return json.dumps(values)

def LoadJSONConf(jsondata, default=None):
    # jsondata must be a json string or dictionary
    # load from json
    # default: the default configuration class to be used if the json values do not
    # specify the class as `ccc`
    if isinstance(jsondata, basestring):
        try:
            jsondata = json.loads(jsondata)
        except:
            return jsondata
    if not isinstance(jsondata, dict):
        return jsondata
    for k,v in jsondata.items():
        jsondata[k] = LoadJSONConf(v, default=default)
        
    confclass = jsondata.get("ccc")
    if not confclass:
        if not default:
            raise ConfigurationError, "Configuration class not found (ccc)"
        return default(**jsondata)
    conf = ResolveName(confclass, base="nive")(**jsondata)
    return conf




def ClassFactory(configuration, reloadClass=False, raiseError=True, base=None):
    """
    Creates a python class reference from configuration. Uses configuration.context as class
    and dynamically adds classes listed as configuration.extensions as base classes.
    
    configuration requires

    - configuration.context
    - configuration.extensions [optional]
    
    If reloadClass = False the class is cached as configuration._v_class.
    """
    if not reloadClass:
        try:
            return configuration._v_class
        except:
            pass
    tag = configuration.context
    if "extensions" in configuration:
        bases = configuration.extensions
    else:
        bases = None
    cls = GetClassRef(tag, reloadClass, raiseError, base)
    if not cls:
        return None
    
    def cacheCls(configuration, cls):
        # store type() class
        lock = configuration.locked
        if lock:
            configuration.unlock()
        configuration._v_class = cls
        if lock:
            configuration.lock()
    
    if not bases:
        cacheCls(configuration, cls)
        return cls

    # load extensions
    b = [cls]
    #opt
    for r in bases:
        r = GetClassRef(r, reloadClass, raiseError, base)
        if not r:
            continue
        b.append(r)
    if len(b)==1:
        return cls

    # create new class with name configuration.context
    cls = type("_factory_"+cls.__name__, tuple(b), {})
    cacheCls(configuration, cls)
    return cls


def GetClassRef(tag, reloadClass=False, raiseError=True, base=None):
    """
    Resolve class reference from python dotted string.
    """
    if isinstance(tag, basestring):
        if raiseError:
            classRef = ResolveName(tag, base=base)
        else:
            try:
                classRef = ResolveName(tag, base=base)
            except ImportError, e:
                return None
        if not classRef:
            return None
        #if reloadClass:
        #    reload(classRef)
        return classRef
    # tag is class ref
    return tag




# test request and response --------------------------------------

class Response(object):
    headerlist = []

class Request(object):
    POST = {}
    GET = {}
    url = ""
    username = ""
    response = Response()
    environ = {}

class Event(object):
    request = Request()

class FakeLocalizer(object):
    def translate(self, text):
        try:
            if text.mapping:
                v = unicode(text)
                for k in text.mapping:
                    v = v.replace(u"${%s}"%k, unicode(text.mapping[k]))
                return v
        except:
            pass
        return text



#-*- coding: utf-8 -*-

import time
import unittest

from nive.definitions import OperationalError
from nive.utils.path import *

from nive.application import *
from nive.definitions import *
from nive.helper import *
from nive.portal import Portal
from nive.security import User

from nive.components.objects.base import ApplicationBase

from nive.tests import __local

# real database test configuration
# change these to fit your system
dbconf = DatabaseConf(
    dbName = __local.ROOT+"nive.db",
    fileRoot = __local.ROOT,
    context = "Sqlite3"
)

root = RootConf(
    id = u"root",
    name = u"Data root",
    default = 1,
    context = "nive.components.objects.base.RootBase",
    subtypes = [IObject]
)

type1 = ObjectConf(
    id = u"type1",
    name = u"Type 1 container",
    hidden = False,
    dbparam = u"data1",
    context = "nive.components.objects.base.ObjectContainerBase",
    subtypes = [IObject]
)
type1.data = [
    FieldConf(id="ftext", datatype="text", size=1000, name=u"ftext", fulltext=1),
    FieldConf(id="fnumber", datatype="number", size=8, name=u"fnumber", required=1),
    FieldConf(id="fdate", datatype="datetime", size=8, name=u"fdate"),
    FieldConf(id="flist", datatype="list", size=100, name=u"flist", listItems=[{"id": u"item 1", "name":u"Item 1"},{"id": u"item 2", "name":u"Item 2"},{"id": u"item 3", "name":u"Item 3"}]),
    FieldConf(id="fmselect", datatype="mselection", size=50, name=u"fmselect"),
    FieldConf(id="funit", datatype="unit", size=8, name=u"funit"),
    FieldConf(id="funitlist", datatype="unitlist", size=100, name=u"funitlist"),
    FieldConf(id="fjson", datatype="json", size=1000, name=u"fjson"),
]

type2 = ObjectConf(
    id = u"type2",
    name = u"Type 2 Object",
    hidden = True,
    dbparam = u"data2",
    context = "nive.components.objects.base.ObjectBase",
    subtypes = None
)
data2 = []
data2.append(FieldConf(**{"id":u"fstr", "datatype": "string", "size": 100, "name": u"fstr", "fulltext": 1}))
data2.append(FieldConf(**{"id":u"ftext", "datatype": "text", "size": 1000, "name": u"ftext"}))
data2.append(FieldConf(**{"id":u"file1", "datatype": "file", "size": 5*1024*1024, "name": u"file1"}))
data2.append(FieldConf(**{"id":u"file2", "datatype": "file", "size": 500*1024, "name": u"file2"}))
type2.data = data2

type3 = ObjectConf(
    id = u"type3",
    name = u"Type 3 Object",
    hidden = True,
    dbparam = u"data3",
    context = "nive.components.objects.base.ObjectContainerBase",
    subtypes = [INonContainer]
)
data3 = []
data3.append(FieldConf(**{"id":u"fstr", "datatype": "string", "size": 100, "name": u"fstr", "fulltext": 1}))
type3.data = data3

cat1 = CategoryConf(**{u"id":u"cat1", u"name":u"Category 1"})
cat2 = CategoryConf(**{u"id":u"cat2", u"name":u"Category 2"})

group1 = GroupConf(**{u"id":u"group1", u"name":u"Group 1"})
group2 = GroupConf(**{u"id":u"group2", u"name":u"Group 2"})

# configuration
appconf = AppConf(
    id = u"unittest",
    context="nive.components.objects.base.ApplicationBase",
    title = u"nive application python unittest",
    modules = [root, type1, type2, type3],
    meta = [FieldConf(id="testfld", datatype="number", size=4, name=u"Number")],
    categories = [cat1, cat2],
    groups = [group1, group2],
    fulltextIndex = True
)


# test data -----------------------------------------------------------------------
data1_1 = { u"ftext": "this is text!",
            u"fnumber": 123456,
            u"fdate": "2008/06/23 16:55:00",
            u"flist": ["item 1", "item 2", "item 3"],
            u"fmselect": "item 5",
            u"funit": 35,
            u"funitlist": [34, 35, 36],
            u"title":"äüöß and others",
            u"pool_type": "type1",
            u"pool_category": "cat1"}
data2_1 = { u"fstr": u"this is sting!",
            u"ftext": u"this is text!",
            u"title": u"äüöß and others",
            u"pool_type": u"type2",
            u"pool_category": u"cat2"}
data3_1 = { u"title": u"title data 3",
            u"fstr": u"testing type 3!"}

data1_2 = { u"ftext": "this is a new text!",
            u"funit": 0,
            u"title":"new title data 1"}
data2_2 = { u"fstr": "this is new sting!",
            u"title": "new title data 2"}

file2_1_data="This is the first text"
file2_2_data=u"This is the text in the second file"
file2_1 = {"filename":"file1.txt", "file":file2_1_data}
file2_2 = {"filename":"file2.txt", "file":file2_2_data}


# empty -------------------------------------------------------------------------
def app_db(modules=None):
    a = ApplicationBase()
    a.Register(appconf)
    a.Register(dbconf)
    if modules:
        for m in modules:
            a.Register(m)
    p = Portal()
    p.Register(a, "nive")
    a.LoadConfiguration()
    dbfile = DvPath(a.dbConfiguration.dbName)
    if not dbfile.IsFile():
        dbfile.CreateDirectories()
    root = DvPath(a.dbConfiguration.fileRoot)
    if not root.IsDirectory():
        root.CreateDirectories()
    
    try:
        a.Query("select id from pool_meta where id=1")
        a.Query("select id from data1 where id=1")
        a.Query("select id from data2 where id=1")
        a.Query("select id from data3 where id=1")
        a.Query("select id from pool_files where id=1")
        a.Query("select id from pool_sys where id=1")
        a.Query("select id from pool_groups where id=1")
    except:
        a.GetTool("nive.components.tools.dbStructureUpdater")()

    # disable this to update test tables each time the tests are called
    #a.GetTool("nive.components.tools.dbStructureUpdater")()
    a.Startup(None)
    # this will reset all testdata
    #emptypool(a)
    return a

def app_nodb():
    a = ApplicationBase()
    a.Register(appconf)
    a.Register(DatabaseConf())
    p = Portal()
    p.Register(a, "nive")
    a.LoadConfiguration()
    try:
        a.Startup(None)
    except OperationalError:
        pass
    return a

def emptypool(app):
    db = app.db
    db.Query(u"delete FROM pool_meta")
    db.Query(u"delete FROM pool_files")
    db.Query(u"delete FROM pool_fulltext")
    db.Query(u"delete FROM pool_groups")
    db.Query(u"delete FROM pool_sys")
    db.Query(u"delete FROM data2")
    db.Query(u"delete FROM data1")
    db.Commit()
    DvDirCleaner(str(db.root)).DeleteFiles(subdirectories = True)
    db.root.CreateDirectories()

def createpool(path,app):
    path.CreateDirectories()
    app.GetTool("nive.components.tools.dbStructureUpdater")()


def statdb(app):
    c = app.db.GetCountEntries()
    countdb = c
    ##print "Count entries in DB:", c
    return c

def root(a):
    r = a.GetRoot("root")
    return r

def createObj1(c):
    type = u"type1"
    data = data1_1
    user = User(u"test")
    o = c.Create(type, data = data, user = user)
    #o.Commit()
    return o

def createObj2(c):
    type = u"type2"
    data = data2_1
    user = User(u"test")
    o = c.Create(type, data = data, user = user)
    #o.Commit()
    return o

def createObj3(c):
    type = u"type3"
    data = data3_1
    user = User(u"test")
    o = c.Create(type, data = data, user = user)
    #o.Commit()
    return o

def createObj1file(c):
    type = "type1"
    data = data1_1.copy()
    data["file1"] = File(**file2_1)
    data["file2"] = File(**file2_2)
    user = User(u"test")
    o = c.Create(type, data = data, user = user)
    #o.Commit()
    return o

def createObj2file(c):
    type = u"type2"
    data = data2_1.copy()
    data["file2"] = File(**file2_1)
    user = User(u"test")
    o = c.Create(type, data = data, user = user)
    #o.Commit()
    return o

def maxobj(r):
    id = a.GetMaxID()
    return r.LookupObj(id)




import time
import unittest

from nive.application import *
from nive.definitions import *
from nive.helper import *
from nive.portal import Portal

from nive.tests.db_app import *


class containerTest_db:
    
    def setUp(self):
        self.app = app_db()
        self.remove=[]

    def tearDown(self):
        u = User(u"test")
        root = self.app.root()
        for r in self.remove:
            root.Delete(r, u)
        self.app.Close()


    def test_basics(self):
        #print "Testing shortcuts"
        a=self.app
        user = User(u"test")
        ccc = a.db.GetCountEntries()
        self.assert_(a.root())
        self.assert_(a.db)
        r = a.root()
        o1 = createObj1(r)
        o2 = createObj2(o1)
        id1 = o1.id
        id2 = o2.id
        self.remove.append(id1)
        self.assert_(a.obj(id1))
        self.assert_(r.__getitem__(str(id1)))
        self.assert_(r.obj(id1))
        self.assert_(o1.obj(id2))
        self.assert_(o1.db==a.db)
        self.assert_(r.db==a.db)
        
        self.assert_(r.IsTypeAllowed("type1", user=user))
        self.assert_(o1.IsTypeAllowed("type1", user=user))
        
        try:
            o2.IsTypeAllowed("type1", user=user)
            self.assert_(False, "ObjectBase is not a container: IsTypeAllowed")
        except AttributeError:
            pass
        self.assert_(r.GetAllowedTypes(user=user))
        self.assert_(o1.GetAllowedTypes(user=user))
        r.Delete(id1, user=user)
        self.assertEqual(ccc, a.db.GetCountEntries())
        

    def test_createroots(self):
        #print "Testing root basics"
        a=self.app
        ccc = a.db.GetCountEntries()
        r=root(a)
        user = User(u"test")
        #rootValues()
        self.assert_(r.GetID()<=0)
        self.assert_(r.GetTypeID())
        self.assert_(r.GetTitle())
        self.assert_(r.GetPath())
        #rootParents()
        self.assert_(r.GetRoot()==r)
        self.assert_(r.GetApp())
        self.assert_(r.GetParent()==None)
        self.assert_(len(r.GetParents())==0)
        self.assert_(len(r.GetParentTitles())==0)
        self.assert_(len(r.GetParentPaths())==0)
        self.assertEqual(ccc, a.db.GetCountEntries())
        

    def test_createobjs(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=root(a)
        # create
        user = User(u"test")
        ccc = a.db.GetCountEntries()
        o1 = createObj1(r)
        self.assert_(o1)
        self.remove.append(o1.id)
        o2 = createObj2(r)
        self.assert_(o2)
        self.remove.append(o2.id)
        o3 = createObj1(o1)
        self.assert_(o3)
        o4 = createObj1(o3)
        self.assert_(o4)
        
        o5 = createObj3(o1)
        self.assert_(o5)
        o6 = createObj2(o5)
        self.assert_(o6)
        self.assertRaises(ContainmentError, createObj1, o5)
        
        self.assert_(ccc+6==statdb(a))
        #Values()
        self.assert_(r.IsContainer())
        self.assert_(o1.IsContainer())
        self.assert_(o2.IsContainer()==False)
        self.assert_(o1.GetID())
        self.assert_(o1.GetTypeID()=="type1")
        self.assert_(o1.GetTitle())
        self.assert_(o1.GetPath())
        #Parents()
        self.assert_(o1.GetRoot()==r)
        self.assert_(o1.GetApp()==a)
        self.assert_(o1.GetParent()==r)
        self.assert_(len(o1.GetParents())==1)
        self.assert_(len(o1.GetParentIDs())==1)
        self.assert_(len(o1.GetParentTitles())==1)
        self.assert_(len(o1.GetParentPaths())==1)

        self.assert_(o4.GetParent()==o3)
        self.assert_(len(o4.GetParents())==3)
        self.assert_(len(o4.GetParentIDs())==3)
        self.assert_(len(o4.GetParentTitles())==3)
        self.assert_(len(o4.GetParentPaths())==3)
        self.assert_(o3.GetObj(o4.GetID()))
        
        newO = r.Duplicate(o1, user)
        self.assert_(newO)
        self.remove.append(newO.id)
        
        r.Delete(o1.GetID(), user=user)
        r.Delete(o2.GetID(), user=user)
        self.assertEqual(ccc+5, a.db.GetCountEntries())
        r.Delete(newO.GetID(), user=user)
        self.assertEqual(ccc, a.db.GetCountEntries())
        

    def test_lists(self):
        #print "Testing objects and subobjects"
        a=self.app
        r=root(a)
        ccc = a.db.GetCountEntries()
        user = User(u"test")
        # errors
        id = 9865898568444
        try:
            r.LookupObj(id)
            self.assert_(False)
        except:
            pass
        try:
            r.GetObj(id)
            self.assert_(False)
        except:
            pass
        # objects and parents
        self.assert_(r.GetSort())
        ccontainer = len(r.GetContainers(batch=False))
        cobjs = len(r.GetObjs(batch=False))
        ccontainer2 = len(r.GetContainerList(parameter={u"pool_type":u"type2"}))
        cobjs2 = len(r.GetObjsList(parameter={u"pool_type":u"type2"}))
        c=statdb(a)
        o1 = createObj1(r)
        self.assert_(o1)
        self.remove.append(o1.id)
        o2 = createObj2(r)
        self.assert_(o2)
        self.remove.append(o2.id)
        o3 = createObj1(o1)
        self.assert_(o3)
        o4 = createObj2(o3)
        self.assert_(o4)
        o5 = createObj2(o3)
        self.assert_(o5)
        self.assert_(c+5==statdb(a))
        try:
            createObj1(o5)
            self.assert_(False)
        except:
            pass
        try:
            createObj2(o5)
            self.assert_(False)
        except:
            pass

        # objects
        id = o1.GetID()
        self.assert_(r.LookupObj(id))
        self.assert_(r.GetObj(id))
        self.assert_(o1.GetObj(o3.GetID()))
        self.assert_(o3.GetObj(o4.GetID()))
        id = o5.GetID()
        self.assert_(r.LookupObj(id))
        self.assert_(r.LookupTitle(id))

        # subitems
        #root
        self.assert_(len(r.GetContainers(batch=False))==ccontainer+2)
        self.assert_(len(r.GetObjs(batch=False))==cobjs+2)
        self.assert_(len(r.GetContainers())==ccontainer+2)  # less failsafe than batch=False. On failure reset testdata.
        self.assert_(len(r.GetObjs())==cobjs+2)
        self.assert_(len(r.GetContainerList(parameter={u"pool_type":u"type2"}))==ccontainer2+1)
        self.assert_(len(r.GetObjsList(parameter={u"pool_type":u"type2"}))==cobjs2+1)
        self.assert_(len(o3.GetObjsBatch([o4.id,o5.id])))
        # object
        self.assert_(len(o3.GetContainers())==2)
        self.assert_(len(o3.GetObjs())==2)
        self.assert_(len(o3.GetContainerList(parameter={u"pool_type":u"type2"}))==2)
        self.assert_(len(o3.GetObjsList(parameter={u"pool_type":u"type2"}))==2)
        self.assert_(len(o3.GetContainerList(parameter={u"pool_type":u"type2"}, operators={"pool_type":u"<>"}))==0)
        self.assert_(len(o3.GetObjsList(parameter={u"pool_type":u"type2"}, operators={u"pool_type":u"<>"}))==0)
        self.assert_(len(o1.GetContainedIDs())==3)
        
        r.DeleteInternal(o1.GetID(), user=user)
        r.DeleteInternal(o2.GetID(), user=user, obj=o2)
        self.assertEqual(ccc, a.db.GetCountEntries())
        


    def test_restraintsConf(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=root(a)
        
        p,o=r.ObjQueryRestraints(self)
        p1,o1=r.queryRestraints
        self.assert_(p==p1)
        self.assert_(o==o1)
        
        r.queryRestraints = {"id":123}, {"id":"="}
        p,o=r.ObjQueryRestraints(self)
        self.assert_(p=={"id":123})
        self.assert_(o=={"id":"="})
        
        r.queryRestraints = {"id":123}, {"id":"="}
        p,o=r.ObjQueryRestraints(self, {"aa":123}, {"aa":"="})
        self.assert_(p=={"id":123,"aa":123})
        self.assert_(o=={"id":"=","aa":"="})
        
        def callback(**kw):
            kw["parameter"]["id"] = 456
            kw["operators"]["id"] = "LIKE"
        r.ListenEvent("loadRestraints", callback)
        p,o=r.ObjQueryRestraints(self)
        self.assert_(p=={"id":456})
        self.assert_(o=={"id":"LIKE"})


    def test_restraintsLookup(self):
        #print "Testing objects and subobjects"
        a=self.app
        r=root(a)
        ccc = a.db.GetCountEntries()
        user = User(u"test")
        # errors
        id = 9865898568444
        try:
            r.LookupObj(id)
            self.assert_(False)
        except:
            pass
        try:
            r.GetObj(id)
            self.assert_(False)
        except:
            pass
        # objects and parents
        self.assert_(r.GetSort())
        ccontainer = len(r.GetContainers(batch=False))
        cobjs = len(r.GetObjs(batch=False))
        ccontainer2 = len(r.GetContainerList(parameter={u"pool_type":u"type2"}))
        cobjs2 = len(r.GetObjsList(parameter={u"pool_type":u"type2"}))
        c=statdb(a)
        o1 = createObj1(r)
        self.assert_(o1)
        self.remove.append(o1.id)
        o2 = createObj2(r)
        self.assert_(o2)
        self.remove.append(o2.id)
        o3 = createObj1(o1)
        self.assert_(o3)
        o4 = createObj2(o3)
        self.assert_(o4)
        o5 = createObj2(o3)
        self.assert_(o5)
        self.assert_(c+5==statdb(a))
        try:
            createObj1(o5)
            self.assert_(False)
        except:
            pass
        try:
            createObj2(o5)
            self.assert_(False)
        except:
            pass

        r.queryRestraints = {"pool_state":99}, {"pool_state":">"}

        # objects
        id = o1.GetID()
        self.assertFalse(r.LookupObj(id))
        self.assertFalse(r.GetObj(id))
        self.assertFalse(o1.GetObj(o3.GetID()))
        self.assertFalse(o3.GetObj(o4.GetID()))
        id = o5.GetID()
        self.assertFalse(r.LookupObj(id))
        self.assertFalse(r.LookupTitle(id))

        # subitems
        #root
        self.assert_(len(r.GetContainers(batch=False))==0)
        self.assert_(len(r.GetObjs(batch=False))==0)
        self.assert_(len(r.GetContainers())==0)
        self.assert_(len(r.GetObjs())==0)
        self.assert_(len(r.GetContainerList(parameter={u"pool_type":u"type2"}))==0)
        p,o=r.ObjQueryRestraints(r)
        p.update({u"pool_type":u"type2"})
        self.assert_(len(r.GetObjsList(parameter=p))==0)
        self.assert_(len(o3.GetObjsBatch([o4.id,o5.id]))==0)
        # object
        self.assert_(len(o3.GetContainers())==0)
        self.assert_(len(o3.GetObjs())==0)
        self.assert_(len(o3.GetContainerList(parameter={u"pool_type":u"type2"}))==0)
        p,o=r.ObjQueryRestraints(self)
        p.update({u"pool_type":u"type2"})
        self.assert_(len(o3.GetObjsList(parameter=p))==0)
        p,o=r.ObjQueryRestraints(self)
        self.assert_(len(o3.GetContainerList(parameter={u"pool_type":u"type2"}, operators={u"pool_type":u"<>"}))==0)
        p,o=r.ObjQueryRestraints(self)
        p.update({u"pool_type":u"type2"})
        o.update({u"pool_type":u"<>"})
        self.assert_(len(o3.GetObjsList(parameter=p, operators=o))==0)
        
        r.queryRestraints = {}, {}

        r.DeleteInternal(o1.GetID(), user=user)
        r.DeleteInternal(o2.GetID(), user=user, obj=o2)
        self.assertEqual(ccc, a.db.GetCountEntries())
        


    def test_shortcuts(self):
        #print "Testing shortcuts"
        a=self.app
        user = User(u"test")
        ccc = a.db.GetCountEntries()
        self.assert_(a.root())
        self.assert_(a.db)
        r = a.root()
        #root
        r.Close()
        r.app
        r.db
        r.root()
        r.GetID()
        r.GetTypeID()
        r.GetTypeName()
        r.GetTitle()
        r.GetPath()
        r.IsRoot()
        r.GetRoot()
        r.GetApp()
        r.GetParent()
        r.GetParents()
        r.GetParentIDs()
        r.GetParentTitles()
        r.GetParentPaths()
        r.GetTool("nive.components.tools.example")


    def test_shortcuts2(self):
        #print "Testing shortcuts"
        a=self.app
        user = User(u"test")
        ccc = a.db.GetCountEntries()
        self.assert_(a.root())
        self.assert_(a.db)
        r = a.root()
        o1 = createObj1(r)
        self.assert_(o1)
        self.remove.append(o1.id)
        #root
        o1.app
        o1.db
        o1.root()
        o1.GetID()
        o1.GetTypeID()
        o1.GetTypeName()
        o1.GetTitle()
        o1.GetPath()
        o1.IsRoot()
        o1.GetRoot()
        o1.GetApp()
        o1.GetParent()
        o1.GetParents()
        o1.GetParentIDs()
        o1.GetParentTitles()
        o1.GetParentPaths()
        o1.GetTool("nive.components.tools.example")
        o1.Close()
        self.assertEqual(ccc+1, a.db.GetCountEntries())
        r.Delete(o1.id, user)

class containerTest_db_(containerTest_db, unittest.TestCase):
    """
    """



class groupsrootTest_db:
    
    def setUp(self):
        self.app = app_db(["nive.components.extensions.localgroups"])
        self.remove=[]

    def tearDown(self):
        u = User(u"test")
        root = self.app.root()
        for r in self.remove:
            root.Delete(r, u)
        pass

    def test_rootsGroups(self):
        a=self.app
        r=root(a)

        userid = u"test"
        r.RemoveLocalGroups(None, None)
        self.assertFalse(r.GetLocalGroups(userid))
        r.AddLocalGroup(userid, u"group:local")
        self.assertItemsEqual(r.GetLocalGroups(userid), [u"group:local"])
        r.RemoveLocalGroups(u"nouser", u"nogroup")
        self.assertItemsEqual(r.GetLocalGroups(userid), [u"group:local"])
        r.RemoveLocalGroups(userid, u"nogroup")
        self.assertItemsEqual(r.GetLocalGroups(userid), [u"group:local"])
        r.RemoveLocalGroups(u"nouser", u"group:local")
        self.assertItemsEqual(r.GetLocalGroups(userid), [u"group:local"])
        r.RemoveLocalGroups(userid, u"group:local")
        self.assertFalse(r.GetLocalGroups(userid))
        r.AddLocalGroup(userid, u"group:local")
        r.RemoveLocalGroups(userid, None)
        self.assertFalse(r.GetLocalGroups(userid))





class groupsrootTest_db_(groupsrootTest_db, unittest.TestCase):
    """
    """


if __name__ == '__main__':
    unittest.main()



import unittest

class TestZPTTemplateLoader(unittest.TestCase):
    def _makeOne(self, **kw):
        from nive.components.reform.template import ZPTTemplateLoader
        return ZPTTemplateLoader(**kw)

    def test_search_path_None(self):
        loader = self._makeOne()
        self.assertEqual(loader.search_path, [])

    def test_search_path_string(self):
        loader = self._makeOne(search_path='path')
        self.assertEqual(loader.search_path, ['path'])

    def test_load_exists(self):
        import os
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        loader = self._makeOne(search_path=[fixtures])
        result = loader.load('test.pt')
        self.failUnless(result)

    def test_load_with_translate(self):
        import os
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        loader = self._makeOne(search_path=[fixtures], translate='abc')
        result = loader.load('test.pt')
        self.assertEqual(result.translate, 'abc')

    def test_load_with_encoding(self):
        import os
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        loader = self._makeOne(search_path=[fixtures], encoding='utf-16')
        result = loader.load('test.pt')
        self.assertEqual(result.encoding, 'utf-16')

    def test_load_with_auto_reload(self):
        import os
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        loader = self._makeOne(search_path=[fixtures], auto_reload=True)
        result = loader.load('test.pt')
        self.assertEqual(result.auto_reload, True)

    def test_load_notexists(self):
        import os
        from nive.components.reform.template import TemplateError
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        loader = self._makeOne(search_path=[fixtures])
        self.assertRaises(TemplateError, loader.load, 'doesnt')
        if hasattr(loader, 'notexists'): # pragma: no cover (chameleon 1)
            self.failUnless(
                os.path.join(fixtures, 'doesnt') in loader.notexists)

    def test_load_negative_cache(self):
        import os
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        path = os.path.join(fixtures, 'test.pt')
        loader = self._makeOne(search_path=[fixtures], auto_reload=True)
        if hasattr(loader, 'notexists'): # pragma: no cover (chameleon 1)
            loader.notexists[path] = True
            result = loader.load('test.pt')
            self.failUnless(result)

    def test_load_negative_cache2(self):
        import os
        from nive.components.reform.template import TemplateError
        fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')
        path = os.path.join(fixtures, 'test.pt')
        loader = self._makeOne(search_path=[fixtures], auto_reload=False)
        if hasattr(loader, 'notexists'): # pragma: no cover (chameleon 1)
            loader.notexists[path] = True
            self.assertRaises(TemplateError, loader.load, 'test.pt')

class TestZPTRendererFactory(unittest.TestCase):
    def _makeOne(self, dirs, **kw):
        from nive.components.reform.template import ZPTRendererFactory
        return ZPTRendererFactory(dirs, **kw)

    def test_functional(self):
        from pkg_resources import resource_filename
        default_dir = resource_filename('nive.components.reform', 'tests/fixtures/')
        renderer = self._makeOne((default_dir,))
        result = renderer('test')
        self.assertEqual(result.strip(), u'<div>Test</div>')

    def test_it(self):
        import os
        path = os.path.join(os.path.dirname(__file__), 'fixtures')
        renderer = self._makeOne(
            (path,),
            auto_reload=True,
            debug=True,
            encoding='utf-16',
            translator=lambda *arg: 'translation',
            )
        template = renderer.load('test')
        self.assertEqual(template.auto_reload, True)
        self.assertEqual(template.debug, True)
        self.assertEqual(template.encoding, 'utf-16')
        self.assertEqual(template.translate('a'), 'translation')

class Test_default_renderer(unittest.TestCase):
    def _callFUT(self, template, **kw):
        from nive.components.reform.template import default_renderer
        return default_renderer(template, **kw)
    
    def test_call_defaultdir(self):
        import re
        result = self._callFUT('checkbox',
                               **{'cstruct':None, 'field':DummyField()})
        result = re.sub('[ \n]+', ' ', result)
        result = re.sub(' />', '/>', result)
        result = result.strip()
        self.assertEqual(result,
                         u'<input type="checkbox" name="name" value="true" '
                         'id="oid"/>')

class DummyWidget(object):
    name = 'name'
    true_val = 'true'
    false_val = 'false'
    css_class = None
    
class DummyField(object):
    widget = DummyWidget()
    name = 'name'
    oid = 'oid'
    required = False
    



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Link
-----
With the link element you can add internal or external links into the web page.

Internal links can be referenced as page id (the number) and are resolved automatically.
"""

from nive.i18n import _
from nive.definitions import StagPageElement, ObjectConf, FieldConf
from nive.components.objects.base import PageElementBase


class link(PageElementBase):
    pass


# text type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "link",
    name = _(u"Link"),
    dbparam = "links",
    context = "nive.cms.link.link",
    template = "link.pt",
    selectTag = StagPageElement,
    icon = "nive.cms.cmsview:static/images/types/link.png",
    description = _(u"With the link element you can add internal or external links into the web page.")
)

targets = [{"id": u"", "name": _(u"Same window")}, {"id": u"blank_", "name": _(u"New window")} ]
styles =  [
    {"id": u"", "name": _(u"none")}, 
    {"id": u"btn", "name": _(u"Simple button")}, 
    {"id": u"btn btn-large", "name": _(u"Large button")}, 
    {"id": u"btn btn-small", "name": _(u"Small button")}, 
]
colors =  [
    {"id": u"", "name": _(u"none")}, 
    {"id": u"btn-primary", "name": _(u"Primary")}, 
    {"id": u"btn-info", "name": _(u"Info")}, 
    {"id": u"btn-success", "name": _(u"Success")}, 
    {"id": u"btn-warning", "name": _(u"Warning")}, 
    {"id": u"btn-danger", "name": _(u"Danger")}, 
    {"id": u"btn-inverse", "name": _(u"Inverse")}, 
]
configuration.data = [
    FieldConf(id="linkurl", datatype="url", size=255, default=u"", name=_(u"Link url"), description=_(u"External links must start with http:// or similar. Internal links can be referenced as page id (the number) and are resolved automatically.")),
    FieldConf(id="target",  datatype="list",size=15,  default=u"", listItems=targets, name=_(u"Target window"), description=u""),
    FieldConf(id="style",   datatype="list",size=20,  default=u"", listItems=styles,  name=_(u"Link style"), description=u""),
    FieldConf(id="color",   datatype="list",size=20,  default=u"", listItems=colors,  name=_(u"Button color"), description=u"")
]

fields = ["title", "linkurl", "target", "style", "color", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []





import copy

import test_MySql
try:
    from nive.utils.dataPool2.mySqlPool import *
except:
    pass

import test_db
from nive.utils.dataPool2.sqlite3Pool import *


mode = "mysql"
printed = [""]

def print_(*kw):
    if type(kw)!=type(""):
        v = ""
        for a in kw:
            v += " "+str(a)
    else:
        v = kw
    if v == "":
        print ".",
        printed.append("")
    else:
        printed[-1] += v
        

def getConnection():
    if mode == "mysql":
        c = MySqlConn(test_MySql.conn, 0)
        print_( "MySQL        -")
    elif mode == "mysqlinno":
        c = test_MySql.conn
        c["dbName"] = u"ut_dataPool2inno"
        c = MySqlConn(c, 0)
        print_( "MySQL InnoDB -")
    else:
        c = Sqlite3Conn(test_db.conn, 0)
        print_( "Sqlite 3     -")
    return c

def getPool():
    if mode == "mysql":
        pool = MySql(test_MySql.conf)
        pool.SetStdMeta(copy.copy(test_MySql.stdMeta))
        pool.GetPoolStructureObj().SetStructure(test_MySql.struct)
        pool.CreateConnection(test_MySql.conn)
        print_( "MySQL        -")
    elif mode == "mysqlinno":
        pool = MySql(test_MySql.conf)
        pool.SetStdMeta(copy.copy(test_MySql.stdMeta))
        pool.GetPoolStructureObj().SetStructure(test_MySql.struct)
        c = test_MySql.conn
        c["dbName"] = u"ut_dataPool2inno"
        pool.CreateConnection(c)
        print_( "MySQL InnoDB -")
    else:
        pool = Sqlite3(test_db.conf)
        pool.SetStdMeta(copy.copy(test_db.stdMeta))
        pool.GetPoolStructureObj().SetStructure(test_db.struct)
        pool.CreateConnection(test_db.conn)
        print_( "Sqlite 3     -")
    return pool

def empty():
    #if mode == "mysql":
    #    test_MySql.emptypool()
    #elif mode == "mysqlinno":
    #    test_MySql.emptypool()
    #else:
    #    t_db.emptypool()
    pass

def connects(n):
    c = getConnection()

    print_( "Connection: ")
    t = time.time()
    for i in range(0,n):
        c.connect()
        c.Close()
    t2 = time.time()
    print_( n, " connects in ", t2-t, "secs. ", (t2-t)/n, " per connect")
    print_()




def cursors(n):
    c = getConnection()
    c.connect()

    print_( "Cursor: ")
    t = time.time()
    for i in range(0,n):
        cu = c.cursor()
        cu.close()
    t2 = time.time()

    c.Close()
    print_( n, " cursors in ", t2-t, "secs. ", (t2-t)/n, " per cursor")
    print_()


def createsql(n):
    pool = getPool()

    print_( "Create SQL: ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta)+list(test_MySql.struct[u"data1"]),
            {u"pool_type": u"data1", u"ftext": u"", u"fnumber": 3},
            sort = u"title, id, fnumber",
            ascending = 0,
            dataTable = u"data1",
            operators={u"pool_type":u"=", u"ftext": u"<>", u"fnumber": u">"},
            start=1,
            max=123)
    t2 = time.time()

    pool.Close()
    print_( n, " sql statements in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery1(n, start):
    pool = getPool()

    print_( "SQL Query data+meta (join no index): ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta)+list(test_MySql.struct[u"data1"]),
            {u"pool_type": u"data1", u"ftext": u"123", u"fnumber": i+start},
            sort = u"title, id, fnumber",
            ascending = 0,
            dataTable = u"data1",
            operators={u"pool_type":u"=", u"ftext": u"LIKE", u"fnumber": u"!="},
            start=1,
            max=123)
        pool.Query(sql, values)
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery2(n, start):
    pool = getPool()

    print_( "SQL Query data+meta=id (join index): ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta)+list(test_MySql.struct[u"data1"]),
            {u"id": i+start},
            sort = u"title",
            ascending = 0,
            dataTable = u"data1",
            operators={},
            start=1,
            max=123)
        pool.Query(sql, values)
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery3(n, start):
    pool = getPool()

    print_( "SQL Query meta=id (index): ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta),
            {u"id": start+i},
            sort = u"id",
            ascending = 0,
            dataTable = u"pool_meta",
            singleTable = 1,
            operators={},
            start=1,
            max=123)
        pool.Query(sql, values)
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery4(n, start):
    pool = getPool()

    print_( "SQL Query meta=id+pool_type=data1 (index): ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta),
            {u"id": start+i, u"pool_type": u"data1"},
            sort = u"id",
            ascending = 0,
            dataTable = u"pool_meta",
            singleTable = 1,
            operators={u"pool_type": u"="},
            start=1,
            max=123)
        pool.Query(sql, values)
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery5(n, start):
    pool = getPool()

    print_( "SQL Query meta=id+pool_type=data1+data.funit (join index): ")
    t = time.time()
    for i in range(0,n):
        sql, values = pool.FmtSQLSelect(list(test_MySql.stdMeta),
            {u"id": start+i, u"pool_type": u"data1", u"funit": 35},
            sort = u"id",
            ascending = 0,
            dataTable = u"data1",
            operators={u"pool_type": u"="},
            start=1,
            max=123)
        pool.Query(sql, values)
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def sqlquery6(n):
    pool = getPool()

    print_( "SQL Query filename (text index): ")
    t = time.time()
    for i in range(0,n):
        files = pool.SearchFilename("file1xxx.txt")
    t2 = time.time()

    pool.Close()
    print_( n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement")
    print_()


def createentries(n):
    pool = getPool()

    print_( "Create entries (nodb): ")
    t = time.time()
    for i in range(0,n):
        e=pool._GetPoolEntry(i, version=None, datatbl=u"data1", preload=u"skip", virtual=True)
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()


def checkentries(n):
    pool = getPool()

    print_( "Create entries (nodb) and check exists: ")
    t = time.time()
    for i in range(0,n):
        e=pool._GetPoolEntry(i, version=None, datatbl=u"data1", preload=u"skip", virtual=True)
        e.Exists()
    t2 = time.time()

    pool.Close()
    print_( n, " checks in ", t2-t, "secs. ", (t2-t)/n, " per check")
    print_()

def createentries2(n):
    pool = getPool()

    print_( "Create entries (nodata): ")
    t = time.time()
    for i in range(0,n):
        e=pool.CreateEntry(u"data1")
        #e.data.update(data1_1)
        #e.meta.update(meta1)
        e.Commit()
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def createentries3(n):
    pool = getPool()

    print_( "Create entries (data+meta): ")
    t = time.time()
    for i in range(0,n):
        e=pool.CreateEntry(u"data1")
        if i==0:  id = e.GetID()
        e.data.update(test_MySql.data1_1)
        e.meta.update(test_MySql.meta1)
        e.Commit()
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()
    return id

def createentries4(n):
    pool = getPool()

    print_( "Create entries (data+meta+file): ")
    t = time.time()
    for i in range(0,n):
        e=pool.CreateEntry("data1")
        if i==0:  id = e.GetID()
        e.data.update(test_MySql.data1_1)
        e.meta.update(test_MySql.meta1)
        e.CommitFile(u"file1", {"file":test_MySql.file1_1, "filename": "file1.txt"})
        e.Commit()
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()
    return id


def getentries1(n, start):
    pool = getPool()

    print_( "Get entries (all): ")
    t = time.time()
    for i in range(0,n):
        e=pool.GetEntry(i+start)
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getentries2(n, start):
    pool = getPool()

    print_( "Get entries (all+file): ")
    t = time.time()
    for i in range(0,n):
        e=pool.GetEntry(i+start, preload=u"all")
        f=e.GetFile(u"file1")
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getentries5(n, start):
    pool = getPool()

    print_( "Get entries (all+filestream): ")
    t = time.time()
    for i in range(0,n):
        e=pool.GetEntry(i+start, preload=u"all")
        #f=e.GetFile("file1")
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getentries4(n, start):
    pool = getPool()

    print_( "Get entries (meta): ")
    t = time.time()
    for i in range(0,n):
        e=pool.GetEntry(i+start, preload=u"meta")
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getbatch1(n, start):
    pool = getPool()

    print_( "Get batch (no preload): ")
    t = time.time()
    ids = []
    for i in range(0,n):
        ids.append(i+start)
    e=pool.GetBatch(ids, preload=u"skip")
    t2 = time.time()

    del e
    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getbatch2(n, start):
    pool = getPool()

    print_( "Get batch (meta): ")
    t = time.time()
    ids = []
    for i in range(0,n):
        ids.append(i+start)
    e=pool.GetBatch(ids, preload=u"meta")
    t2 = time.time()

    del e
    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def getbatch3(n, start):
    pool = getPool()

    print_( "Get batch (all): ")
    t = time.time()
    ids = []
    for i in range(0,n):
        ids.append(i+start)
    e=pool.GetBatch(ids, preload=u"all")
    t2 = time.time()

    del e
    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()

def delentries(n, start):
    pool = getPool()

    print_( "Delete entries (meta+data): ")
    t = time.time()
    for i in range(0,n):
        pool.DeleteEntry(i+start)
        pool.Commit()
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()


def delentries2(n, start):
    pool = getPool()

    print_( "Delete entries (meta+data+file): ")
    t = time.time()
    for i in range(0,n):
        pool.DeleteEntry(i+start)
        pool.Commit()
    t2 = time.time()

    pool.Close()
    print_( n, " entries in ", t2-t, "secs. ", (t2-t)/n, " per entry")
    print_()


def report(modes, printed):
    rep=[]
    c = len(printed)/len(modes)
    for n in range(0, c):
        p = 0
        for m in modes:
            rep.append(printed[p*c+n])
            p+=1
    print 
    print
    i=0
    for p in rep:
        print p
        i+=1
        if i==len(modes):
            print
            i=0


def run(modes):
    global mode , printed
    n = 1000
    printed = [""]
    for m in modes:
        mode = m
        print
        print mode,
        empty()
        connects(n)
        cursors(n)
        createsql(n)
        createentries(n)
        checkentries(n)
        createentries2(n)
        id = createentries3(n)
        id2 = createentries4(n)
        getentries1(n, id2)
        getentries2(n, id2)
        getentries5(n, id2)
        getentries4(n, id2)
        getbatch1(n, id2)
        getbatch2(n, id2)
        getbatch3(n, id2)
        sqlquery1(n, id2)
        sqlquery2(n, id)
        sqlquery3(n, id2)
        sqlquery4(n, id)
        sqlquery5(n, id2)
        sqlquery6(n)
        delentries(n, id)
        delentries2(n, id2)
    report(modes, printed)


if __name__ == '__main__':
    #run(("sqlite3",))
    run(("sqlite3","mysql","mysqlinno"))



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
"""

from nive.definitions import IPage
from nive.workflow import WfProcessConf, WfStateConf, WfTransitionConf
from nive.i18n import _


# change pool_state to publish or revoke
def publish(transition, context, user, values):
    context.meta.set("pool_state", 1)
    
def revoke(transition, context, user, values):
    context.meta.set("pool_state", 0)


wfProcess = WfProcessConf(
    id = "default",
    name = _(u"Simple Publishing Workflow"),
    apply = (IPage,),
    states = (
        WfStateConf(
            id = "start",
            name = _(u"New object"),
            actions = ("delete","duplicate","edit","add","remove")
        ),
        WfStateConf(
            id = "edit",
            name = _(u"Edit"),
            actions = ("delete","duplicate","edit","add","remove")
        ),
        WfStateConf(
            id = "public",
            name = _(u"Public"),
            actions = ("delete","duplicate","edit","add","remove")
        )
    ),
    transitions = (
        WfTransitionConf(
            id = "create",
            name = _(u"Create"),
            fromstate = "start",
            tostate = "edit",
            roles = ("group:editor","group:author"),
            actions = ("create",),
            execute = (revoke,)
        ),
        WfTransitionConf(
            id = "publish",
            name = _(u"Publish"),
            fromstate = "edit",
            tostate = "public",
            roles = ("group:editor","group:reviewer"),
            actions = ("publish",),
            execute = (publish,)
        ),
        WfTransitionConf(
            id = "revoke",
            name = _(u"Revoke"),
            fromstate = "public",
            tostate = "edit",
            roles = ("group:editor","group:reviewer"),
            actions = ("revoke",),
            execute = (revoke,)
        )
    )    
)




import binascii
import os
import getpass
import uuid
import base64

from pyramid.compat import native_

from pyramid.scaffolds import PyramidTemplate # API
from pyramid.scaffolds.template import Template

class DefaultSqliteTemplate(PyramidTemplate):
    _template_dir = 'defaultWebsite'
    summary = 'A simple Nive cms website with Sqlite database'

    def pre(self, command, output_dir, vars):
        """ Overrides :meth:`pyramid.scaffold.template.Template.pre`, adding
        several variables to the default variables list (including
        ``random_string``, and ``package_logger``).  It also prevents common
        misnamings (such as naming a package "site" or naming a package
        logger "root".
        """
        # configuration
        user = None
        while not user:
            user = raw_input("Admin username: ")
        vars['adminuser'] = user
    
        pprompt = lambda: (getpass.getpass('Password: '), getpass.getpass('Retype password: '))
    
        p1, p2 = pprompt()
        while p1 != p2:
            print('Passwords do not match. Try again')
            p1, p2 = pprompt()
        
        vars['adminpass'] = base64.standard_b64encode(p1)

        mail = raw_input("Admin email: ")
        vars['adminemail'] = str(mail)
        
        vars['root'] = raw_input("Root directory for files (default data): ")
        if not vars['root']:
            vars['root'] = "data"

        vars['comment'] = "# SQLite"
        vars['param_website'] = """context="Sqlite3", dbName="%(root)s/website.db" """ % vars
        vars['param_user'] = """context="Sqlite3", dbName="%(root)s/userdb.db" """ % vars
        vars['dbPackage'] = ''
        
        vars['language'] = raw_input("Locale name. Please choose english-> en or german-> de: ")
        
        vars['authsecret'] = str(uuid.uuid4())
        vars['cookiesecret'] = str(uuid.uuid4())

        return PyramidTemplate.pre(self, command, output_dir, vars)
    
    
class DefaultMysqlTemplate(PyramidTemplate):
    _template_dir = 'defaultWebsite'
    summary = 'A simple Nive cms website with Sqlite database'

    def pre(self, command, output_dir, vars):
        """ Overrides :meth:`pyramid.scaffold.template.Template.pre`, adding
        several variables to the default variables list (including
        ``random_string``, and ``package_logger``).  It also prevents common
        misnamings (such as naming a package "site" or naming a package
        logger "root".
        """
        # configuration
        user = None
        while not user:
            user = raw_input("Admin username: ")
        vars['adminuser'] = user
        
        pprompt = lambda: (getpass.getpass('Password: '), getpass.getpass('Retype password: '))
    
        p1, p2 = pprompt()
        while p1 != p2:
            print('Passwords do not match. Try again')
            p1, p2 = pprompt()
            
        vars['adminpass'] = base64.standard_b64encode(p1)
        
        mail = raw_input("Admin email: ")
        vars['adminemail'] = str(mail)

        print "Please enter MySql database settings for the website. Two databases will be used, one for website contents ",
        print "and one to store userdata. Host, port, user and password are used for both databases."
        print ""
        print "The cms will create all required database tables and columns automatically when starting the webserver."
        print "These manual administrator actions are required after running this installation script:"
        print "- Create the two databases"
        print "- Assign create and alter table rights to the database user"
        
        vars['root'] = raw_input("Root directory for files (default data): ")
        if not vars['root']:
            vars['root'] = "data"
            
        vars['dbcontentname'] = raw_input("Content database name (default %s): " % vars["project"])
        if not vars['dbcontentname']:
            vars['dbcontentname'] = vars["project"]
        vars['dbusername'] = raw_input("User database name (default %s_user): "% vars["project"])
        if not vars['dbusername']:
            vars['dbusername'] = vars["project"]+"_user"
        vars['dbhost'] = raw_input("MySql database host (default localhost): ")
        vars['dbport'] = raw_input("MySql database port (leave empty to use default): ")
        vars['dbuser'] = raw_input("MySql database user: ")
            
        pprompt = lambda: (getpass.getpass('Password: '), getpass.getpass('Retype password: '))
    
        p1, p2 = pprompt()
        while p1 != p2:
            print('Passwords do not match. Try again')
            p1, p2 = pprompt()

        vars['dbpass'] = base64.standard_b64encode(p1)
        
        vars['language'] = raw_input("Locale name. Please choose english-> en or german-> de: ")

        vars['comment'] = "# MySql "
        vars['param_website'] = """context="MySql",
            dbName="%(dbcontentname)s",
            host="%(dbhost)s",
            port="%(dbport)s",
            user="%(dbuser)s",
            password=base64.decodestring("%(dbpass)s") """ % vars

        vars['param_user'] = """context="MySql",
            dbName="%(dbusername)s",
            host="%(dbhost)s",
            port="%(dbport)s",
            user="%(dbuser)s",
            password=base64.decodestring("%(dbpass)s") """ % vars
        vars['dbPackage'] = "'MySQL-python'"

        vars['authsecret'] = str(uuid.uuid4())
        vars['cookiesecret'] = str(uuid.uuid4())

        return PyramidTemplate.pre(self, command, output_dir, vars)




import time
import unittest

from nive.definitions import *
from nive.tools import *
from nive.security import User 

from nive.components.tools.example import configuration
import db_app

# -----------------------------------------------------------------

class ToolTest1(unittest.TestCase):

    def test_tool(self):
        t=Tool(ToolConf(id="test",data=[FieldConf(id="test",datatype="string")], values={"test":"aaaaa"}), None)
        t.Run()
        self.assert_(t.ExtractValues()["test"]=="aaaaa")
        t.AppliesTo("type1")
        self.assert_(t.GetAllParameters())
        self.assert_(t.GetParameter("test"))


class ToolTest(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_db()

    def tearDown(self):
        self.app.Close()

    def test_toolapp(self):
        t = self.app.GetTool("nive.components.tools.example")
        self.assert_(t)
        r,v = t()
        self.assert_(r)

    def test_toolapp2(self):
        self.app.Register("nive.components.tools.example")
        t = self.app.GetTool("exampletool")
        self.assert_(t)
        r,v = t()
        self.assert_(r)

    def test_toolapp3(self):
        c2 = ToolConf(configuration)
        c2.id = "exampletool2"
        c2.apply = (IApplication,)
        self.app.Register(c2)
        t = self.app.GetTool("exampletool2", self.app)
        self.assert_(t)
        r,v = t()
        self.assert_(r)

    def test_toolobj(self):
        user = User(u"test")
        r = self.app.root()
        o = db_app.createObj1(r)
        t = o.GetTool("nive.components.tools.example")
        r1,v = t()
        self.assert_(r1)
        r.Delete(o.GetID(), user=user)


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
nive Tool base class to use as base for new tools. 

Tools represent an executeable function to be run with or without an
object context. Tools can be loaded by dotted python name or, if registered
by app.Register(), with their id. 
Registering tools is necessary to set application specific values and to 
automatically apply tools to objects.

Tools can be applied to selected object types or interfaces or global. They are always 
loaded by *id*. ::

    tool = app().GetTool("sendMail")
    tool(recv="me@domain.com",title="New mail")

Tool return values on execution are :: 

    result, stream/data = tool()

*stream/data* depends on the tool. By default tools use StringIO to write output data. *result*
signals whether the tool succeeded or not.
 
Parameter value lookup on execution:

1) ``tool.Run(**values)``: execution specific values
2) ``tool.configuration.values``: configured values on application level
3) ``tool.data``: default configured prarameter values

To create a new tool:
Subclass and overwrite ``def _Run()`` function. Use self.stream to write result data.

"""

from StringIO import StringIO

from nive.definitions import Interface, ITool, implements
from nive.views import BaseView
from nive.forms import ToolForm


class Tool(object):
    """

    """
    implements(ITool)

    def __init__(self, configuration, app):
        self.configuration = configuration
        self.app_ = app
        self.stream = False

        self.__name__ = ""
        self.__parent__ = None
        self.__acl__ = []

        self.id = ""
        self.mimetype = "text/html"
        self._LoadConfiguration()


    def _LoadConfiguration(self):
        """
        loads self.configuration 
        """
        if not self.configuration:
            raise ConfigurationError, "Tool configuration is None. Please load the tool by referencing the tool configuration."
        
        c = self.configuration
        for k in c.keys():
            # special values
            if k == "id" and c.id:
                self.__name__ = c.id
                self.id = c.id
            if k == "acl" and c.acl:
                self.__acl__ = c.acl
                continue
            # map value
            setattr(self, k, c[k])
            
    @property
    def app(self):
        """ returns the cms application object """
        return self.app_
    

    # Subclass functions --------------------------------------------

    def _Run(self, **values):
        result = True
        return result


    # Execution --------------------------------------------------------------------------------------------

    def __call__(self, **kw):
        return self.Run(**kw)


    def Run(self, **kw):
        """
        Execute the tool.
        
        If stream is disabled (self.stream = None) and no stream is passed,
        the function returns the result data as string/binary.
        Otherwise the stream is returned as second parameter.
        
        returns bool, stream
        """
        if not self.stream:
            self.stream = StringIO()

        # call function
        values = self.ExtractValues(**kw)
        values["original"] = kw
        result = self._Run(**values)
        return result, self.stream


    def ExtractValues(self, **kw):
        """
        Extract values for configured parameters

        1) tool.Run(values): call specific values
        2) tool.configuration.values: configured values on application level
        3) tool.data: default prarameter values
        
        returns dictionary
        """
        values = {}
        cv = self.configuration.values
        for p in self.configuration.data:
            if   p.id in kw:      values[p.id] = kw[p.id]
            elif p.id in cv:      values[p.id] = cv[p.id]
            else:                 values[p.id] = p.default
        return values


    def AppliesTo(self, poolType):
        """
        Return if this tool applies to the given nive object type.
        
        returns bool
        """
        if not self.configuration.apply:
            return False
        return poolType in self.configuration.apply


    def GetAllParameters(self):
        """
        Return the configured parameter definitions for the function
        
        returns list
        """
        return self.configuration.data


    def GetParameter(self, id):
        """
        Get single parameter definition for function
        
        returns configuration or None
        """
        for fld in self.configuration.data:
            if fld.id == id:
                return fld
        return None



class _IGlobal(Interface):
    """
    used for global tool registration as tool.apply
    """

class _GlobalObject(object):
    """
    used for global tool lookup
    """
    implements(_IGlobal)
    
    
    
class ToolView(BaseView):    
    """
    """
    
    def form(self):
        """
        Run a tool by rendering the default form and execute on submit.
        This function does not return a valid Response object. This view is meant to
        be called from another view or template:
        ``view.RenderView(tool)``
        """
        form = ToolForm(view=self, loadFromType=self.context.configuration)
        form.Setup()
        result, data, action = form.Process()
        if not isinstance(data, basestring):
            try:
                data = data.getvalue()
            except:
                data = str(data)
        if result and action.id == "run" and self.context.mimetype != "text/html":
            fn = None
            if hasattr(self.context, "filename"):
                fn = self.context.filename
            return self.SendResponse(data=data, mime=self.context.mimetype, raiseException=True, filename=fn)
        return self.SendResponse(form.HTMLHead() + data, mime=self.context.mimetype, raiseException=False) 
    
    
    
    
           



import time
import unittest
import gc

from nive.definitions import OperationalError
from nive.application import *
from nive.definitions import *
from nive.helper import *
from nive.events import Events
from nive.portal import Portal

from nive.tests import db_app

mApp = AppConf(id="app", groups=[GroupConf(id="g1",name="G1")], categories=[CategoryConf(id="c1",name="C1")])
dbConfiguration = DatabaseConf(dbName="test")

mObject = ObjectConf(id="object", dbparam="object", name="Object",
                     data=(FieldConf(id="a1",datatype="string",name="A1"),FieldConf(id="a2",datatype="number",name="A2"),))

mRoot = RootConf(id="root")
mTool = ToolConf(id="tool", context="nive.components.tools.example")
mViewm = ViewModuleConf(id="vm")
mView = ViewConf(id="v",context=mApp,view=mApp)
mMod = ModuleConf(id="mod", context=mApp)



mApp2 = AppConf(id="app2", 
                context="nive.tests.test_nive.testapp",
                modules=[mObject,mRoot,mTool,mViewm,mView,mMod], 
                groups=[GroupConf(id="g1",name="G1")], 
                categories=[CategoryConf(id="c1",name="C1")])
dbConfiguration2 = DatabaseConf(dbName="test")



class testapp(Application, Registration, Configuration, AppFactory, Events):
    """
    """
    
def app():
    app = testapp()
    app.Register(mApp2)
    app.Register(dbConfiguration2)
    app.Startup(None)
    portal = Portal()
    portal.Register(app, "nive")
    return app



class baseTest(unittest.TestCase):
    
    def setUp(self):
        pass
    def tearDown(self):
        pass


    def test_referencing(self):
        pass
        #a = app()
        #r = a.root()
        #a.portal.Close()
        


class dbTest(unittest.TestCase):

    def setUp(self):
        pass
    def tearDown(self):
        pass


    def test_database(self):
        user = User(u"test")
        app = db_app.app_db()
        r = app.root()

        type = u"type1"
        data = db_app.data1_1
        o = r.Create(type, data = data, user = user)
        self.assert_(o)
        id = o.id
        #print "get_referents:"
        #for x in gc.get_referents(o):
        #    print x
        #print
        #print "get_referrers:"
        #for x in gc.get_referrers(o):
        #    print x
        del o
        
        o=app.obj(id, rootname = "")
        self.assert_(o)
        r.Delete(id, user=user)
        del o
        #app.portal.Close()
        print 
        a=1


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
Root for context to run adminview 
"""

from nive.definitions import RootConf
from nive.userdb.root import root
from nive.i18n import _

class adminroot(root):
	"""
	"""



# Root definition ------------------------------------------------------------------
#@nive_module
configuration = RootConf(
	id = "usermanagement",
	context = "nive.userdb.useradmin.adminroot.adminroot",
    default = False,
	subtypes = "*",
	name = _(u"User listing"),
	description = ""
)





import unittest

from nive.definitions import *

from nive.components.extensions.cache import *

from nive.tests import db_app


class testobj(object, ContainerCache):
    type = "object"
    def GetTypeID(self):
        return self.type
        
        

class ContainerCache(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    
    def test_1(self):
        obj = testobj()
        obj2 = testobj()
        
        self.assert_(obj.Cache(obj2, "cid"))
        self.assert_(obj.GetFromCache("cid")==obj2)
        self.assert_(len(obj.GetAllFromCache())==1)
        self.assert_(obj.RemoveCache("cid"))
        self.assert_(len(obj.GetAllFromCache())==0)
        self.assert_(obj.GetFromCache(id="cid")==None)


    def test_2(self):
        obj = testobj()
        obj.cacheTypes = ["object"]
        obj2 = testobj()
        obj3 = testobj()
        obj3.type = "aaaaa"
        
        self.assert_(obj.Cache(obj2, "cid"))
        self.assert_(obj.GetFromCache("cid")==obj2)
        self.assert_(len(obj.GetAllFromCache())==1)
        self.assert_(obj.RemoveCache("cid"))
        self.assert_(len(obj.GetAllFromCache())==0)
        self.assert_(obj.GetFromCache(id="cid")==None)
        
        
    def test_3(self):
        obj = testobj()
        obj.cacheTypes = ["object"]
        obj2 = testobj()
        obj2.type = "aaaaa"
        
        self.assertFalse(obj.Cache(obj2, "cid"))
        self.assert_(obj.GetFromCache("cid")!=obj2)
        self.assert_(len(obj.GetAllFromCache())==0)




import time
import unittest

from nive.i18n import *


class i18nTest(unittest.TestCase):

    def test_translate(self):
        t = translate("a nice string")
        self.assert_(t=="a nice string")

    def test_translator(self):
        t = translator()
        self.assert_(t("a nice string")=="a nice string")


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Root for public user functions
-------------------------------------------
Provides functionality to safely add users with several activation and 
mailing options.
"""

import types, base64, random, string
from time import time
import uuid
import json

from nive.definitions import RootConf, Conf, StagUser, IUser
from nive.definitions import Interface, implements
from nive.security import User, AdminUser, IAdminUser, UserFound
from nive.components.objects.base import RootBase
from nive.i18n import _


class root(RootBase):
    """
    """
    
    # field used as unique user identity in sessions and acache
    identityField = u"name"
    
    # User account handling ------------------------------------------------------------------------------------------------------

    def AddUser(self, data, activate=1, generatePW=0, mail=None, notify=False, notifyMail=None, groups="", currentUser=None, **kw):
        """
        Create a new user with groups for login with name/password ::

            data: user data as dictionary. groups and pool_state are removed. 
            activate: directly activate the user for login (pool_state=1)
            generatePW: generate a random password to be send by mail
            mail: mail object template for confirmation mail
            notify: notify user admin on new registrations by mail
            notifyMail: mail object template for notify mail
            groups: initially assign groups to the user
            currentUser: the currently logged in user for pool_createdby and workflow
    
        returns tuple: the user object if succeeds and report list
        """
        report = []
        name = data.get("name")
        
        if not name or name == "":
            report.append(_(u"Please enter your username"))
            return None, report

        # check user with name exists
        user = self.GetUserByName(name, activeOnly=0)
        if user:
            report.append(_(u"Username '${name}' already in use. Please choose a different name.", mapping={u"name":name}))
            return None, report
        email = data.get("email")
        if email and self.app.configuration.get("loginByEmail"):
            user = self.GetUserByMail(email, activeOnly=0)
            if user:
                report.append(_(u"Email '${name}' already in use. ", mapping={'name':email}))
                return None, report
        
        if generatePW:
            pw = self.GeneratePassword()
            data["password"] = pw

        if groups:
            data["groups"] = groups

        data["pool_type"] = u"user"
        data["pool_state"] = int(activate)
        data["pool_stag"] = StagUser

        if not currentUser:
            currentUser = User(name)
        obj = self.Create("user", data=data, user=currentUser)
        if not obj:
            report.append(_(u"Sorry. Account could not be created."))
            return None, report
        #obj.Commit(currentUser)
        
        app = self.app
        if mail:
            title = mail.title
            body = mail(user=obj, **kw)
            tool = app.GetTool("sendMail")
            try:
                result, value = tool(body=body, title=title, recvids=[str(obj)], force=1)
                if not result:
                    report.append(_(u"The email could not be sent."))
                    return None, report
            except Exception, e:
                report.append(_(u"The email could not be sent."))
                report.append(str(e))
                #report.append(_("Send Mail Error: ")+str(e))
                return None, report

        sysadmin = app.configuration.get("systemAdmin")
        if notify and sysadmin:
            if notifyMail:
                title = notifyMail.title
                body = notifyMail(user=obj)
            else:
                title = app.configuration.title + u" - New user"
                body = u"""
                User: %s<br/>
                Mail: %s<br/>
                Name: %s %s<br/>
                Groups: %s<br/>
                Active: %s<br/>
                <br/>%s<br/>
                """ % (data.get("name",u""), data.get("email",u""), data.get("surname",u""), data.get("lastname",u""), 
                       data.get("groups",u""), data.get("pool_state",u""), data.get("comment",u""))
            tool = app.GetTool("sendMail")
            try:
                result, value = tool(body=body, title=title, recvmails=[sysadmin], force=1)
            except Exception, e:
                pass

        report.append(_(u"Account created."))
        return obj, report


    def MailUserPass(self, email=None, mailtmpl=None, createNewPasswd=True, currentUser=None):
        """
        Mails a new password or the current password in plain text.
        
        returns status and report list
        """
        report=[]

        if not email:
            report.append(_(u"Please enter your email address or username."))
            return False, report

        obj = self.GetUserByMail(email)
        if not obj:
            obj = self.GetUserByName(email)
            if not obj:
                report.append(_(u"No matching account found. Please try again."))
                return False, report

        email = obj.data.get("email")
        title = obj.meta.get("title")
        name = obj.data.get("name")
        if email == "":
            report.append(_("No email address found."))
            return False, report
        recv = [(email, title)]

        if createNewPasswd:
            pwd = self.GenerateID(5)
            obj.data["password"] = pwd
            obj.Commit(user=currentUser)
        else:
            pwd = obj.data.get("password")
        if mailtmpl:
            title = mailtmpl.title
            body = mailtmpl(user=obj, password=pwd)
        else:
            title = _("Password")
            body = pwd
        tool = self.app.GetTool("sendMail")
        try:
            result, value = tool(body=body, title=title, recvmails=recv, force=1)
            if not result:
                report.append(_(u"The email could not be sent."))
                return False, report
        except Exception, e:
            report.append(_(u"The email could not be sent."))
            #report.append("Send Mail Error: "+str(e))
            return False, report
        report.append(_(u"The new password has been sent to your email address. Please sign in and change it."))
        return True, report


    # Login/logout and user sessions ------------------------------------------------------------------------------------------------------

    def Login(self, name, password, raiseUnauthorized = 1):
        """
        returns user/none and report list
        """
        report = []

        # session login
        user = self.GetUserByName(name)
        if not user:
            if raiseUnauthorized:
                raise Unauthorized, "Login failed"
            report.append(_(u"Sign in failed. Please check your username and password."))
            return None, report
            
        if not user.Authenticate(password):
            if raiseUnauthorized:
                raise Unauthorized, "Login failed"
            report.append(_(u"Sign in failed. Please check your username and password."))
            return None, report

        # call user
        user.Login()
        report.append(_(u"You are now signed in."))
        return user, report


    def Logout(self, ident):
        """
        Logout and delete session data
        """
        user = self.GetUser(ident)
        if not user:
            return False
        if not IUser.providedBy(user):
            user = self.LookupUser(id=user.id)
        if user:
            user.Logout()
        return True


    # Password, activationID ------------------------------------------------------------------------------------------------------

    def GenerateID(self, length=20, repl="-"):
        # generates a id
        return str(uuid.uuid4()).replace(repl,"")[:length]
        

    def GeneratePassword(self, mincount=5, maxcount=5):
        # generates a password
        vowels = u"aeiou0123456789#*"
        consonants = u"bcdfghjklmnpqrstvwxyz"
        password = u""

        for x in range(1,random.randint(mincount+1,maxcount+1)):
            if random.choice([1,0]):
                consonant = consonants[random.randint(1,len(consonants)-1)]
                if random.choice([1,0]):
                    consonant = consonant.upper()
                password=password + consonant
            else:
                vowel = vowels[random.randint(1,len(vowels)-1)]
                password=password + vowel

        return password


    # User ------------------------------------------------------------------------------------------------------

    def GetUser(self, ident, activeOnly=1):
        """
        Lookup user by *user identity* as used in session cookies for example
        
        events: 
        - getuser(ident, activeOnly)
        - loaduser(user)
        """
        try:
            self.Signal("getuser", ident=ident, activeOnly=activeOnly)
        except UserFound, user:
            return user.user
        user = self.LookupUser(ident=ident, activeOnly=activeOnly)
        if user:
            self.Signal("loaduser", user=user)
        return user
    

    def GetUserByName(self, name, activeOnly=1):
        """ """
        return self.LookupUser(name=name, activeOnly=activeOnly)


    def GetUserByMail(self, email, activeOnly=1):
        """ """
        return self.LookupUser(email=email, activeOnly=activeOnly)


    def GetUserByID(self, id, activeOnly=1):
        """ """
        return self.LookupUser(id=id, activeOnly=activeOnly)


    def LookupUser(self, id=None, ident=None, name=None, email=None, activeOnly=1, reloadFromDB=0):
        """ 
        reloadFromDB deprecated. will be removed in the future
        """
        if not id:
            loginByEmail = self.app.configuration.get("loginByEmail")
            # lookup adminuser
            admin = self.app.configuration.get("admin")
            if admin:
                if ident and ident == admin.get(self.identityField):
                    return AdminUser(admin, admin[self.identityField])
                elif name and name == admin["name"]:
                    return AdminUser(admin, admin.get(self.identityField))
                elif loginByEmail and email and email == admin["email"]: 
                    return AdminUser(admin, admin.get(self.identityField))
            # lookup id for name, email or ident
            param = {}
            if activeOnly:
                param[u"pool_state"] = 1
            if name:
                param[u"name"] = name
            elif email:
                param[u"email"] = email
            elif ident:
                if not self.identityField:
                    raise ValueError, "user identity filed not set"
                param[self.identityField] = ident
                
            user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
            
            # check multiple identity fields
            if len(user)==0 and loginByEmail:
                if name:
                    del param[u"name"]
                    param[u"email"] = name
                    user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
                elif email:
                    del param[u"email"]
                    param[u"name"] = email
                    user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
                else:
                    return None
            
            if len(user)!=1:
                return None
            id = user[0][0]
            
        user = self.GetObj(id)
        if not user or (activeOnly and not user.meta.get("pool_state")==1):
            return None
        return user
     

    def GetUsers(self, **kw):
        """
        """
        return self.SearchType(u"user", {u"pool_state":1}, [u"id",u"title",u"name",u"groups",u"lastlogin"])


    def GetUserInfos(self, userids, fields=None, activeOnly=True):
        """
        """
        param = {self.identityField:userids}
        if activeOnly:
            param[u"pool_state"] = 1
        if not fields:
            fields = [u"id", u"name", u"email", u"title", u"groups", self.identityField]
        return self.SelectDict(pool_type=u"user", parameter=param, fields=fields, operators={self.identityField:u"IN"})
    
    
    def GetUsersWithGroup(self, group, fields=None, activeOnly=True):
        """
        """
        param = {u"groups":group}
        if activeOnly:
            param[u"pool_state"] = 1
        operators = {u"groups": "LIKE"}
        if not fields:
            fields = [u"name",u"groups"]
        elif not u"groups" in fields:
            fields = list(fields)
            fields.append(u"groups")
        users = self.SelectDict(pool_type=u"user", parameter=param, fields=fields, operators=operators)
        # verify groups
        verified = []
        for u in users:
            try:
                g = json.loads(u["groups"])
            except:
                continue
            if group in g:
                verified.append(u)
        return verified


    def DeleteUser(self, ident, currentUser=None):
        """
        returns status and report list
        """
        report = []
        if isinstance(ident, basestring):
            if not ident:
                report.append(_(u"Invalid user."))
                return False, report
    
            user = self.LookupUser(ident=ident, activeOnly=0)
            if not user:
                report.append(_(u"Invalid username."))
                return False, report
        else:
            user = ident

        if IAdminUser.providedBy(user):
            report.append(_(u"You cannot delete the admin user."))
            return False, report
            
        if not self.Delete(user.id, obj=user, user=currentUser):
            report.append(_(u"Sorry. An error occured."))
            return False, report

        report.append(_(u"User deleted."))
        return True, report


    # to be removed ------------------------------------------------------
    def GetUserGroups(self, name, activeOnly=1):
        """
        """
        user = self.GetUser(name, activeOnly=activeOnly)
        if not user:
            return None
        return user.data.groups


    def Encrypt(self, string):
        try:
            return base64.encodestring(string)
        except:
            return string

    def Decrypt(self, string):
        try:
            return base64.decodestring(string)
        except:
            return string




# Root definition ------------------------------------------------------------------

#@nive_module
configuration = RootConf(
    id = "udb",
    context = "nive.userdb.root.root",
    template = "root.pt",
    views = ["nive.userdb.userview.view"],
    default = 1,
    subtypes = "*",
    name = _(u"User account"),
    description = __doc__
)


from nive.components.reform.schema import Invalid
from nive.components.reform.schema import Email

def UsernameValidator(node, value):
    """ 
    Validator which succeeds if the username does not exist.
    Can be used for the name input field in a sign up form.
    """
    # lookup name in database
    r = node.widget.form.context.root()
    u = r.LookupUser(name=value, activeOnly=0)
    if u:
        # check if its the current user
        ctx = node.widget.form.context
        if len(u)==1 and ctx.id == u[0][0]:
            return
        err = _(u"Username '${name}' already in use. Please choose a different name.", mapping={'name':value})
        raise Invalid(node, err)

def EmailValidator(node, value):
    """ 
    Validator which succeeds if the email does not exist.
    Can be used for the email input field in a sign up form.
    """
    # validate email format
    Email()(node, value)
    # lookup email in database
    r = node.widget.form.context.root()
    u = r.LookupUser(email=value, activeOnly=0)
    if u:
        # check if its the current user
        ctx = node.widget.form.context
        if len(u)==1 and ctx.id == u[0][0]:
            return
        err = _(u"Email '${name}' already in use. Please use the login form if you already have a account.", mapping={'name':value})
        raise Invalid(node, err)




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import types


from nive.tools import Tool
from nive.definitions import ToolConf, FieldConf, IApplication, MetaTbl, Structure
from nive.i18n import _

configuration = ToolConf()
configuration.id = "dbSqldataDump"
configuration.context = "nive.components.tools.dbSqldataDump.dbSqldataDump"
configuration.name = _(u"Database sql dump")
configuration.description = _("This function only dumps table contents and skips 'create table' statements.")
configuration.apply = (IApplication,)
configuration.data = [
    FieldConf(id="excludeSystem", datatype="mcheckboxes", default=[], listItems=[{"id":"pool_sys", "name":"pool_sys"},{"id":"pool_fulltext","name":"pool_fulltext"}], 
              name=_(u"Exclude system columns"))
]
configuration.mimetype = "text/sql"



class dbSqldataDump(Tool):
    """
    """

    def _Run(self, **values):

        result = 1
        codepage="utf-8"
    
        app = self.app
        datapool = app.db
        conf = app.dbConfiguration
        conn = datapool.connection
        system = values.get("excludeSystem")
        self.filename = app.configuration.id + ".sql"

        if not conn:
            self.stream.write(_(u"Database connection error (${name})\n", mapping={u"name": app.poolTag}))
            return 0
        
        if not conn.IsConnected():
            self.stream.write(_(u"Database connection error (${name})\n", mapping={u"name": app.poolTag}))
            return 0
        
        def mapfields(fields):
            a=[]
            for f in fields:
                a.append(f.id)
            return a
        
        export = [(MetaTbl,mapfields(app.GetAllMetaFlds(ignoreSystem=False)))]
        for t in app.GetAllObjectConfs():
            export.append((t.dbparam, ["id"]+mapfields(t.data)))
        for t in Structure.items():
            export.append((t[0], mapfields(t[1]["fields"])))

        for table in export:
            #tablename
            tablename=table[0]
            if system and tablename in system:
                continue 
            #fields
            fields=table[1]
            columns = (",").join(fields)
            sql="select %s from %s" % (columns, tablename)
            c = conn.cursor()
            c.execute(sql)
            for rec in c.fetchall():
                data = []
                for col in rec:
                    data.append(conn.FmtParam(col))
                data = (",").join(data)
                value = u"INSERT INTO %s (%s) VALUES (%s);\n"%(tablename, columns, data)
                value = value.encode(codepage)
                self.stream.write(value)        
        
        return 1




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Note
----
Simple text note for authors. The note is only visible to cms authors and not published.
"""

from nive.i18n import _
from nive.definitions import StagPageElement, ObjectConf, FieldConf
from nive.components.objects.base import PageElementBase
from nive.utils.utils import ConvertHTMLToText, CutText

class note(PageElementBase):

    titleLen = 20

    def Init(self):
        self.ListenEvent("commit", "TextToTitle")

    def TextToTitle(self):
        text = ConvertHTMLToText(self.data.get("textblock"), removeReST=True)
        self.meta["title"] = CutText(text, self.titleLen, postfix=u"")
        return True
    


# note type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "note",
    name = _(u"Note"),
    dbparam = "notes",
    context = "nive.cms.note.note",
    template = "note.pt",
    selectTag = StagPageElement,
    icon = "nive.cms.cmsview:static/images/types/note.png",
    description = _(u"Simple text note for authors. The note is only visible to cms authors and not published.")
)

configuration.data = [
    FieldConf(id="textblock", datatype="htext", size=50000, default=u"", name=_(u"Note"), description=u"")
]

fields = ["textblock", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []




import time
import unittest

from nive.definitions import *
from nive.workflow import *
from nive.security import User

from nive.tests.db_app import app_db



def wftestfunction(transition, context, user, values):
    context.testvalue = "huh"

def wfcallbackfunction(transition, context, user, values):
    return True
def wfcallbackfunction0(transition, context, user, values):
    return False



s1 = WfStateConf(
    id = "start",
    name = "Entry point",
    actions = ["create", "delete"]
)
s2 = WfStateConf(
    id = "edit",
    name = "Edit",
    actions = ["edit", "delete"]
)
s3 = WfStateConf(
    id = "end",
    name = "End",
    actions = []
)


t1 = WfTransitionConf(
    id = "create",
    name = "Create",
    fromstate = "start",
    tostate = "edit",
    roles = ("Authenticated",),
    actions = ("create",),
    conditions = (wfcallbackfunction,),
)
t2 = WfTransitionConf(
    id = "edit",
    name = "Edit",
    fromstate = "end",
    tostate = "edit",
    roles = ("Authenticated",),
    actions = ("edit",),
    conditions = (wfcallbackfunction,),
)
t3 = WfTransitionConf(
    id = "commit",
    name = "Commit",
    fromstate = "edit",
    tostate = "end",
    roles = (),
    actions = ("commit",),
    execute = (wftestfunction,)
)
t4 = WfTransitionConf(
    id = "error",
    name = "Error",
    fromstate = "edit",
    tostate = "end",
    roles = (),
    actions = ("error",),
    conditions = (wfcallbackfunction0,),
    execute = (wftestfunction,)
)
t5 = WfTransitionConf(
    id = "multiple1",
    name = "M 1",
    fromstate = "start",
    tostate = "edit",
    roles = ("Authenticated",),
    actions = ("multiple",),
)
t6 = WfTransitionConf(
    id = "multiple2",
    name = "M 2",
    fromstate = "start",
    tostate = "end",
    roles = ("Authenticated",),
    actions = ("multiple",),
)

tx1 = WfTransitionConf(
    id = "error",
    name = "Error",
    fromstate = "edit",
    tostate = "end",
    roles = "*",
    actions = ("error",),
    conditions = ("nive.tests.test_workflow.wfcallbackfunction",wfcallbackfunction,wfcallbackfunction0),
    execute = (wftestfunction,)
)
tx2 = WfTransitionConf(
    id = "error",
    name = "Error",
    fromstate = "edit",
    tostate = "end",
    roles = "*",
    actions = ("error",),
)
tx3 = WfTransitionConf(
    id = "error",
    name = "Error",
    fromstate = "edit",
    tostate = "end",
    roles = (),
    actions = ("error",),
    execute = (wftestfunction,)
)


wf1 = WfProcessConf(
    id = "wf1",
    name = "Workflow 1",
    states = [s1,s2,s3],
    transitions = [t1,t2,t3,t4,t5,t6],
    apply=(IObject,)
)
wf2 = WfProcessConf(
    id = "wf2",
    name = "Workflow 2",
    states = [s1,s2,s3],
    transitions = [t1,t2,t3,t4],
    apply=None
)



class Tmeta(object):
    pool_wfa=""
    pool_wfp = "wf1"
    
class TestO(object):
    implements(IObject)
    def __init__(self):
        self.meta=Tmeta()
        self.id=123
        self.testvalue = ""

    def GetWfState(self):
        return self.meta.pool_wfa
    def SetWfState(self, stateID):
        self.meta.pool_wfa = stateID

class TApp(object):
    def __init__(self):
        self.reloadExtensions=False

class TestU1(object):
    def GetGroups(self, o):
        return ["Authenticated"]
class TestU2(object):
    def GetGroups(self, o):
        return ["group:admin"]



class WfTest(unittest.TestCase):

    def setUp(self):
        self.wf = Process(wf1, TApp())
        self.obj = TestO()
        self.user1 = TestU1()
        self.user2 = TestU2()
    
    def tearDown(self):
        pass
    

    def test_proc1(self):
        self.assert_(self.wf.Allow("create", self.obj, self.user1, transition=None))
        self.assert_(self.wf.Action("create", self.obj, self.user1, transition=None))
        self.assert_(self.obj.meta.pool_wfa=="edit")
        # based on users
        self.assertFalse(self.wf.PossibleTransitions("edit", action="", transition="", context=self.obj, user=self.user1))
        self.assert_(self.wf.PossibleTransitions("edit", action="", transition="", context=self.obj, user=self.user2))

        self.assert_(self.wf.GetObjInfo(self.obj, self.user1, extended = True).get("name")=="Workflow 1")
        self.assert_(self.wf.GetObjState(self.obj).id=="edit")
        self.assert_(self.wf.GetTransition("commit").id=="commit")
        self.assert_(self.wf.GetState("edit").name=="Edit")


    def test_proc2(self):
        self.assert_(self.wf.Allow("create", self.obj, self.user1, transition=None))
        a = self.obj.meta.pool_wfa
        self.obj.meta.pool_wfa = "aaaaa"
        self.assertFalse(self.wf.Allow("create", self.obj, self.user1, transition=None))
        self.obj.meta.pool_wfa = a
        self.assertFalse(self.wf.Allow("ohno", self.obj, self.user1, transition=None))
        self.assert_(self.wf.Action("create", self.obj, self.user1, transition=None))
        try:
            self.wf.Action("commit", self.obj, self.user1, transition=None)
            self.assert_(False, "WorkflowNotAllowed not raised")
        except WorkflowNotAllowed:
            pass
        self.wf.Action("commit", self.obj, self.user2, transition=None)
        self.assert_(self.obj.meta.pool_wfa=="end")
        try:
            self.wf.Action("commit", self.obj, self.user2, transition=None)
            self.assert_(False, "WorkflowNotAllowed not raised")
        except WorkflowNotAllowed:
            pass
        self.wf.Action("edit", self.obj, self.user1, transition=None)
        try:
            self.wf.Action("commit", self.obj, self.user2, transition="edit")
            self.assert_(False, "WorkflowNotAllowed not raised")
        except WorkflowNotAllowed:
            pass
        self.wf.Action("commit", self.obj, self.user2, transition="commit")

    
    def test_multiple(self):
        self.assert_(self.wf.Action("multiple", self.obj, self.user1, transition=None))

    
    def test_wfstates(self):
        self.assert_(self.wf.GetObjState(self.obj).id=="start")
        self.assert_(self.wf.GetObjState(None).id=="start")
        self.assertFalse(self.wf.GetState("ohno"))
        self.assertFalse(self.wf._LoadStates(None))
        self.assertRaises(ConfigurationError, self.wf._LoadStates, [s1,t1])
        

    def test_wftransition(self):
        self.assert_(self.wf.Action("create", self.obj, self.user1, transition=None))
        self.assert_(self.obj.testvalue=="")
        self.assert_(self.wf.Action("commit", self.obj, self.user2, transition=None))
        self.assert_(self.obj.testvalue=="huh")
        self.assert_(self.wf.Action("edit", self.obj, self.user2, transition=None))
        
        self.assertFalse(self.wf.GetTransition("ohno"))
        self.assertFalse(self.wf.GetTransition("commit").IsInteractive())
        self.assertFalse(self.wf.GetTransition("commit").GetInteractiveFunctions())
        try:
            self.wf.Action("error", self.obj, self.user2, transition=None)
            self.assert_(False, "WorkflowNotAllowed not raised")
        except WorkflowNotAllowed:
            pass

        self.assertFalse(self.wf._LoadTransitions(None))
        self.assertRaises(ConfigurationError, self.wf._LoadTransitions, [s1,t1])


    def test_include_app(self):
        a = app_db()
        a.Register(wf1)
        a.Register(wf2)
        self.assert_(a.GetWorkflowConf(wf1.id, self.obj)[0].id==wf1.id)
        self.assert_(a.GetWorkflowConf(wf2.id)[0].id==wf2.id)


class TransisiotnTest(unittest.TestCase):

    def setUp(self):
        self.wf = Process(wf1, TApp())
        self.obj = TestO()
        self.user1 = TestU1()
        self.user2 = TestU2()
    
    def tearDown(self):
        pass
    
    def test_calls1(self):
        trans = Transition("t", tx1, self.wf)
        trans.Allow(self.obj, self.user1)
        trans.IsInteractive()
        trans.GetInteractiveFunctions()
        trans.Execute(self.obj, self.user1, {"a":1})

    def test_calls2(self):
        trans = Transition("t", tx2, self.wf)
        trans.Allow(self.obj, self.user1)
        trans.IsInteractive()
        trans.GetInteractiveFunctions()
        trans.Execute(self.obj, None)
        
    def test_calls3(self):
        trans = Transition("t", tx3, self.wf)
        trans.Allow(self.obj, None)
        trans.IsInteractive()
        trans.GetInteractiveFunctions()
        trans.Execute(self.obj, self.user2)


class ConfTest(unittest.TestCase):

    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    

    def test_conf(self):
        self.assertFalse(len(wf1.test()))
        self.assertFalse(len(s1.test()))
        self.assertFalse(len(s2.test()))
        self.assertFalse(len(s3.test()))
        self.assertFalse(len(t1.test()))
        self.assertFalse(len(t2.test()))
        self.assertFalse(len(t3.test()))
        self.assertFalse(len(t4.test()))
    
    
    def test_obj1(self):
        testconf = WfProcessConf(
            id = "aadd",
            name = "Aadd",
            description = "",
            states = [],
            transitions = []
        )
        self.assert_(len(testconf.test())==0)

    def test_err1(self):
        testconf = WfProcessConf(
            id = "",
            context = "ohno",
            name = "Aadd",
            description = "",
            states = [WfTransitionConf()],
            transitions = [WfStateConf()]
        )
        self.assert_(len(testconf.test())==4)
        self.assert_(str(testconf))
        testconf(self)



    def test_obj2(self):
        testconf = WfTransitionConf(
            id = "ddaa",
            name = "Ddaa",
            description = "",
            fromstate = "here",
            tostate = "there",
            roles = ["master","slave"],
            actions = ["go"],
            execute = ["nive.tests.test_workflow.wftestfunction"],
            conditions = ["nive.tests.test_workflow.wftestfunction"]
        )
        self.assert_(len(testconf.test())==0)

    def test_err2(self):
        testconf = WfTransitionConf(
            id = "",
            context = "ohno",
            name = "Ddaa",
            description = "",
            fromstate = "",
            tostate = "",
            roles = ["master","slave"],
            actions = ["go"],
            execute = ["ohno"],
            conditions = ["ohno"]
        )
        self.assert_(len(testconf.test())==6)
        self.assert_(str(testconf))


    def test_obj3(self):
        testconf = WfStateConf(
            id = "qqqq",
            name = "Qqqq",
            description = "",
            actions = ["stay"]
        )
        self.assert_(len(testconf.test())==0)

    def test_err3(self):
        testconf = WfStateConf(
            id = "",
            context = "ohno",
            name = "Qqqq",
            description = "",
            actions = ["stay"]
        )
        self.assert_(len(testconf.test())==2)
        self.assert_(str(testconf))




if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Image
-----
The image element inserts images into the web page.

Images uploaded as fullsize will be be linked as pop ups.

If the Python Image Library (PIL) is installed automated image conversion on upload can be
activated by adding `nive.components.extensions.images.ImageProcessor` to configuration.extensions.
::

    ProfileImage = Conf(source="imagefull", dest="image", format="JPEG", 
                        quality="85", width=360, height=0, extension="jpg", 
                        condition=CheckDeafult)
    ProfileTeaser = Conf(source="imagefull", dest="image", format="JPEG", 
                         quality="85", width=260, height=0, extension="jpg", 
                         condition=CheckTeaser)
    ProfileTeaserSmall = Conf(source="imagefull", dest="image", format="JPEG", 
                              quality="90", width=160, height=0, extension="jpg", 
                              condition=CheckTeaserSmall)
    
    configuration.imageProfiles = [ProfileImage, ProfileTeaser, ProfileTeaserSmall]

The default image settings for conversions. 
"""

from nive.i18n import _
from nive.definitions import StagPageElement, ObjectConf, FieldConf, Conf
from nive.components.objects.base import PageElementFileBase

from nive.components.extensions.images import PILloaded

class image(PageElementFileBase):
    
    def Span(self):
        # css class span for the css selection
        if self.data.cssClass=="teaserl":
            return u"span4"
        elif self.data.cssClass=="teasers":
            return u"span2"
        return u"span3"
    


# image type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "image",
    name = _(u"Image"),
    dbparam = "images",
    context = "nive.cms.image.image",
    template = "image.pt",
    selectTag = StagPageElement,
    extensions = [],
    icon = "nive.cms.cmsview:static/images/types/image.png",
    description = _(u"The image element inserts images into the web page.")
)

css = [{"id": u"default", "name": _(u"Simple")}, 
       {"id": u"left",    "name": _(u"Float right")}, 
       {"id": u"teaser",  "name": _(u"Teaser")},
       {"id": u"teaserl", "name": _(u"Teaser large")},
       {"id": u"teasers", "name": _(u"Teaser small")},
]

configuration.data = [
    FieldConf(id="image",     datatype="file", size=0,     default=u"", name=_(u"Imagefile")),
    FieldConf(id="imagefull", datatype="file", size=0,     default=u"", name=_(u"Imagefile fullsize")),
    FieldConf(id="textblock", datatype="htext",size=100000,default=u"", name=_(u"Text"), fulltext=1, required=0),
    FieldConf(id="cssClass",  datatype="list", size=10,    default=u"", name=_(u"Styling"), listItems=css),
    FieldConf(id="link",      datatype="url",  size=1000,  default=u"", name=_(u"Link"))
]

if PILloaded and "nive.components.extensions.images.ImageProcessor" in configuration.extensions:
    fields = ["title", "imagefull", "textblock", "cssClass", "link", "pool_groups"]
else:
    fields = ["title", "image", "imagefull", "textblock", "cssClass", "link", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []


# image profiles
def CheckDeafult(imageObject):
    return imageObject.data.cssClass in (u'', u'default', u'left', u'teaserl')

def CheckTeaser(imageObject):
    return imageObject.data.cssClass == u'teaser'

def CheckTeaserSmall(imageObject):
    return imageObject.data.cssClass == u'teasers'

ProfileImage = Conf(source="imagefull", dest="image", format="JPEG", quality="90", width=360, height=0, extension="jpg", condition=CheckDeafult)
ProfileTeaser = Conf(source="imagefull", dest="image", format="JPEG", quality="90", width=260, height=0, extension="jpg", condition=CheckTeaser)
ProfileTeaserSmall = Conf(source="imagefull", dest="image", format="JPEG", quality="90", width=160, height=0, extension="jpg", condition=CheckTeaserSmall)

configuration.imageProfiles = [ProfileImage, ProfileTeaser, ProfileTeaserSmall]






import time
import unittest
from types import DictType

from nive.definitions import *
from zope.interface.registry import * 
from zope.interface import Interface, Attribute, implements, alsoProvides, Provides



# -----------------------------------------------------------------
class ITest(Interface):
    pass
class ITest2(Interface):
    pass
class ITestaaaa(Interface):
    pass

class Test(object):
    Provides(ITest)
    def __call__(self, context):
        return self, context

class Test2(object):
    implements(ITest)

# -----------------------------------------------------------------
testconf = ObjectConf(
    id = "text",
    name = "Text",
    dbparam = "texts",
    context = "nive.tests.thelper.text",
    selectTag = 1,
    description = ""
)
configuration = testconf



class registryTest(unittest.TestCase):

    def test_init1(self):
        registry = Components()
        #registerUtility(self, component=None, provided=None, name=u'', info=u'', event=True, factory=None)

        t2 = Test2()
        alsoProvides(t2, ITest2)
        self.assert_(ITest2.providedBy(t2))
        ITest.providedBy(Test())
        registry.registerUtility(Test(), provided=ITest, name='testconf')

        ITest2.providedBy(Test())
        registry.registerUtility(Test2(), name='testconf2')
        
        self.assert_(registry.queryUtility(ITestaaaa)==None)
        self.assert_(registry.queryUtility(ITest, name=u'testconf'))
        for u in registry.getUtilitiesFor(ITest):
            self.assert_(u)
        self.assert_(registry.getAllUtilitiesRegisteredFor(ITest))
        
        
    def test_init2(self):
        registry = Components()
        IConf.providedBy(Conf())
        registry.registerUtility(Conf(), provided=IConf, name='testconf2')
        
        IObjectConf.providedBy(testconf)
        registry.registerUtility(testconf, provided=IObjectConf, name='testconf3')
        
        registry.registerUtility(configuration, provided=IObjectConf, name='testconf')
        
        
        
    def test_init3(self):
        registry = Components()
        registry.registerAdapter(Test(), (IConf,), ITest, name='testadapter')
        registry.registerAdapter(Test(), (IConf,), ITest, name='testadapter2')
        registry.registerAdapter(Test(), (IConf,), ITest)
        c = Conf()
        
        self.assert_(IConf.providedBy(c))
        #registry.getAdapter(c, ITest, name='testadapter')
        self.assert_(registry.getAdapter(c, ITest))
        
        self.assertFalse(registry.queryAdapter(c, ITest, name='nix'))
        self.assert_(registry.queryAdapter(c, ITest, name='testadapter'))
        a = list(registry.getAdapters((c,), ITest))
        self.assert_(len(a)==3,a)
        

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Nive cms default configuration

This file links together all modules included in a default nive installation.

Minimal local system configuration (sqlite example) and usage
------------------------------------------------------
::
    
    from nive.definitions import AppConf, DatabaseConf

    app = AppConf("nive.cms.app",
                  id = "website",
                  title = "My website",
                  dbConfiguration = DatabaseConf(
                       fileRoot="/var/opt/website",
                       context=u"Sqlite3",
                       dbName="/var/opt/website/website.db")
    )
    design = ViewModuleConf("nive.cms.design.view")
    app.modules.append(design)

- Groups: *group:editor, group:author, group:admin*
- Additional meta fields: *Permission (pool_groups)* Object only displayed to users in 
  the selected group

A design is not included by default. The default can simply be included with the 
following line ::

    app.modules.append("nive.cms.design.view")

To include a customized copy of the design use ::

    design = ViewModuleConf("nive.cms.design.view")
    # design customizations here
    design.static = "mywebsite:static" 
    # add to website
    app.modules.append(design)

This will replace the static directory of the design with your own directory. However, now you will
have to add the required css, images and javascript used by the templates to the new folder.
(For a start just copy the contents of ``nive.cms.design:static``.)
"""

from nive.i18n import _
from nive.definitions import implements, IWebsite
from nive.definitions import AppConf, FieldConf, GroupConf
from nive.security import ALL_PERMISSIONS, Allow, Everyone, Deny
from nive.components.objects.base import ApplicationBase

#@nive_module
configuration = AppConf(
    id = "website",
    title = u"Nive cms",
    context = "nive.cms.app.WebsitePublisher",
    workflowEnabled = True,
    columns=[u"footer"]
)
configuration.modules = [
    # objects
    "nive.cms.box", "nive.cms.column", "nive.cms.menublock", "nive.cms.file", 
    "nive.cms.image", "nive.cms.media", "nive.cms.note", "nive.cms.text",
    "nive.cms.news", "nive.cms.spacer", "nive.cms.link", "nive.cms.code", 
    # page, root
    "nive.cms.root", "nive.cms.page", 
    # cms editor
    "nive.cms.cmsview",
    # design: not included by default
    #"nive.cms.design.view"
    # workflow
    "nive.cms.workflow",
    "nive.cms.workflow.wf.wfProcess", 
    #extensions
    "nive.components.extensions.fulltextpage", 
    #"nive.components.extensions.localgroups",
    # tools
    "nive.components.tools.dbStructureUpdater", "nive.components.tools.dbSqldataDump", "nive.components.tools.cmsstatistics",
    "nive.components.tools.gcdump",
    # administration and persistence
    "nive.adminview",
    "nive.components.extensions.persistence.dbPersistenceConfiguration"
]

configuration.meta = [
    FieldConf(id="pool_groups", datatype="mcheckboxes", size=250, default="", 
              name=_(u"Permission"), description=_(u"Only displayed to users in the selected group"))
]

configuration.acl = [
    (Allow, Everyone, 'view'),
    (Allow, 'group:editor', 'read'),
    (Allow, 'group:editor', 'add'),
    (Allow, 'group:editor', 'edit'), 
    (Allow, 'group:editor', 'delete'), 
    (Allow, 'group:editor', 'publish'), 
    (Allow, 'group:editor', 'revoke'), 
    (Allow, 'group:author', 'read'),
    (Allow, 'group:author', 'add'),
    (Allow, 'group:author', 'edit'), 
    (Allow, 'group:author', 'delete'), 
    (Allow, 'group:reviewer', 'read'),
    (Allow, 'group:reviewer', 'publish'),
    (Allow, 'group:reviewer', 'revoke'), 
    (Allow, 'group:reader', 'read'),
    (Allow, 'group:admin', ALL_PERMISSIONS), 
    (Deny, Everyone, ALL_PERMISSIONS),
]

configuration.groups = [ 
    GroupConf(id="group:editor", name="group:editor"),
    GroupConf(id="group:author", name="group:author"),
    GroupConf(id="group:reviewer", name="group:reviewer"),
    GroupConf(id="group:reader", name="group:reader"),
    GroupConf(id="group:admin",  name="group:admin"),
]


class WebsitePublisher(ApplicationBase):
    """ the main cms application class """
    implements(IWebsite)
    
    
